# Dialogs.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2000 - 2002   Michele Campeotto <micampe@micampe.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import string
import os

import gnome.ui
import gnome.uiconsts

import GDK
import GTK
import GTKSCINTILLA

import Moleskine
import Languages
import findtools
from GladeDialog import GladeDialog

search_dialog = None
search_in_files_dialog = None
replace_dialog = None
options_dialog = None
langs_dialog = None

def _parse_color(color):
    import types
    if type(color) == types.IntType:
        b = (color & 0xFF0000) >> 16
        g = (color & 0x00FF00) >> 8
        r = (color & 0x0000FF)
        return (r, g, b)
    elif type(color) == types.TupleType:
        return color[2] * 65536 + color[1] * 256 + color[0]
    else:
        raise TypeError, '`color\' must be tuple or integer.'

class OptionsDialog(GladeDialog):
    def __init__(self):
        GladeDialog.__init__(self, 'options_dialog')
        self.set_transient_for(Moleskine.app)
        prefs = Moleskine.app.prefs
    
    # Misc
        if prefs['complete/auto'] is not None:
            self.auto_complete.set_active(int(prefs['complete/auto']))
        self.save_win_size.set_active(int(prefs['geometry/save_win_size']))
        self.save_session.set_active(int(prefs['history/save_session']))
    # History
        self.mru_size.set_value(int(prefs['history/mru_size']))
        self.mru_sort.set_active(int(prefs['history/mru_sort']))
    # Default style
        self.def_font.set_font_name(prefs['default style/font'])
        (r, g, b) = _parse_color(int(prefs['default style/fore']))
        self.def_fore.set_i8(r, g, b, 0xFF)
        (r, g, b) = _parse_color(int(prefs['default style/back']))
        self.def_back.set_i8(r, g, b, 0xFF)
    # Line numbers style
        self.linen_font.set_font_name(prefs['line numbers/font'])
        (r, g, b) = _parse_color(int(prefs['line numbers/fore']))
        self.linen_fore.set_i8(r, g, b, 0xFF)
        (r, g, b) = _parse_color(int(prefs['line numbers/back']))
        self.linen_back.set_i8(r, g, b, 0xFF)
    # Matched braces style
        (r, g, b) = _parse_color(int(prefs['braces style/fore_ok']))
        self.brace_ok.set_i8(r, g, b, 0xFF)
        (r, g, b) = _parse_color(int(prefs['braces style/fore_bad']))
        self.brace_bad.set_i8(r, g, b, 0xFF)
    # Selection color
        (r, g, b) = _parse_color(int(prefs['default style/selection']))
        self.selection_colour.set_i8(r, g, b, 0xFF)
    # Bookmark color
        (r, g, b) = _parse_color(int(prefs['default style/bookmark']))
        self.bookmark_colour.set_i8(r, g, b, 0xFF)
    # Caret style
        (r, g, b) = _parse_color(int(prefs['caret/fore']))
        self.caret_fore.set_i8(r, g, b, 0xFF)
        self.caret_width.set_value(int(prefs['caret/width']))
        self.caret_period.set_value(int(prefs['caret/period']))
    # Keyboard
        keys = int(prefs['keyboard/keybindings'])
        if keys == 2:
            self.keys_emacs.set_active(1)
        elif keys == 1:
            self.keys_gnome.set_active(1)
        else:
            self.keys_moleskine.set_active(1)
    # Line numbers digits
        self.show_linen.set_active(int(prefs['line numbers/show']))
        if int(prefs['line numbers/auto']):
            self.digits_auto.set_active(1)
            self.digits_num.set_sensitive(0)
        else:
            self.digits_fixed.set_active(1)
            self.digits_num.set_sensitive(1)
        self.digits_num.set_value(int(prefs['line numbers/digits']))
        
        for dirname in Moleskine.app.prefs.dirnames:
            self.dirnames_list.append(dirname)
    
    def digits_fixed_toggled(self, w, user_data=None):
        if w.get_active():
            self.digits_num.set_sensitive(1)
        else:
            self.digits_num.set_sensitive(0)
    
    def new_dir_clicked(self, w, user_data=None):
        row = self.dirnames_list.append(('', ''))
        self.dirnames_list.select_row(row, 0)
        self.name_entry.set_text('')
        self.dir_entry.set_text('')
        self.name_entry.grab_focus()
    
    def del_dir_clicked(self, w, user_data=None):
        row = self.dirnames_list.selection[0]
        self.dirnames_list.remove(row)
        self.dirnames_list.select_row(row - 1, 0)
    
    def edit_name(self, w, user_data=None):
        if len(self.dirnames_list.selection) > 0:
            row = self.dirnames_list.selection[0]
            self.dirnames_list.set_text(row, 0, w.get_text())
    
    def edit_dir(self, w, user_data=None):
        if len(self.dirnames_list.selection) > 0:
            row = self.dirnames_list.selection[0]
            self.dirnames_list.set_text(row, 1, w.get_text())
    
    def dir_selected(self, clist, row, column, event, user_data=None):
        name = clist.get_text(row, 0)
        self.name_entry.set_text(name)
        directory = clist.get_text(row, 1)
        self.dir_entry.set_text(directory)
        
        self.name_entry.set_sensitive(1)
        self.dir_entry.set_sensitive(1)
        self.del_dir.set_sensitive(1)
    
    def dir_unselected(self, clist, row, column, event, user_data=None):
        self.name_entry.set_text('')
        self.dir_entry.set_text('')
        
        self.name_entry.set_sensitive(0)
        self.dir_entry.set_sensitive(0)
        self.del_dir.set_sensitive(0)
    
    def apply(self):
        prefs = Moleskine.app.prefs
        
        prefs['complete/auto'] = self.auto_complete.get_active()
    # History
        prefs['geometry/save_win_size'] = self.save_win_size.get_active()
        prefs['history/save_session'] = self.save_session.get_active()
        prefs['history/mru_size'] = self.mru_size.get_value_as_int()
        prefs['history/mru_sort'] = self.mru_sort.get_active()
    # Default style
        prefs['default style/font'] = self.def_font.get_font_name()
        prefs['default style/fore'] = \
                _parse_color(self.def_fore.get_i8())
        prefs['default style/back'] = \
                _parse_color(self.def_back.get_i8())
    # Line numbers style
        prefs['line numbers/font'] = self.linen_font.get_font_name()
        prefs['line numbers/fore'] = \
                _parse_color(self.linen_fore.get_i8())
        prefs['line numbers/back'] = \
                _parse_color(self.linen_back.get_i8())
    # Matched braces style
        prefs['braces style/fore_ok'] = \
                _parse_color(self.brace_ok.get_i8())
        prefs['braces style/fore_bad'] = \
                _parse_color(self.brace_bad.get_i8())
    # Selection color
        prefs['default style/selection'] = \
                _parse_color(self.selection_colour.get_i8())
    # Bookmark color
        prefs['default style/bookmark'] = \
                _parse_color(self.bookmark_colour.get_i8())
    # Caret style
        prefs['caret/fore'] = \
                _parse_color(self.caret_fore.get_i8())
        prefs['caret/width'] = self.caret_width.get_value_as_int()
        prefs['caret/period'] = self.caret_period.get_value_as_int()
    # Keyboard
        if self.keys_emacs.get_active():
            prefs['keyboard/keybindings'] = 2
        elif self.keys_gnome.get_active():
            prefs['keyboard/keybindings'] = 1
        else:
            prefs['keyboard/keybindings'] = 0
    # Line numbers digits
        prefs['line numbers/show'] = self.show_linen.get_active()
        prefs['line numbers/auto'] = self.digits_auto.get_active()
        prefs['line numbers/digits'] = self.digits_num.get_value_as_int()
    # Directory names
        Moleskine.app.prefs.dirnames = []
        for row in range(self.dirnames_list.rows):
            name = self.dirnames_list.get_text(row, 0)
            directory = self.dirnames_list.get_text(row, 1)
            
            directory = os.path.expanduser(directory)
            directory = os.path.expandvars(directory)
            directory = os.path.abspath(directory)
            
            Moleskine.app.prefs.dirnames.append((name, directory))
        
        for doc in Moleskine.app.documents:
            doc.setup_line_numbers_digits()
            doc.setup_braces_style()
            doc.update_title()
        Moleskine.app.update_mru_menu()
        Moleskine.app.update_styles()
        Moleskine.app.update_title()
        Moleskine.app.prefs.save()
    
    def destroy(self, w, user_data=None):
        global options_dialog
        options_dialog = None
    
    def clicked(self, w, button, user_data=None):
        if button == 0:
            self.apply()
            self.close()
        elif button == 1:
            self.apply()
        else:
            self.close()

class LanguagesDialog(GladeDialog):
    def __init__(self):
        GladeDialog.__init__(self, 'languages_dialog')
        self.set_transient_for(Moleskine.app)
        
        self.styles_list.set_auto_sort(1)
        
        lexers = Languages.Lexers.keys()
        lexers.sort()
        self.lexer_combo.set_popdown_strings(lexers)
        
        lang_names = Moleskine.app.langs.keys()
        lang_names.sort()
        self.lang_name_combo.set_popdown_strings(lang_names)
        doc = Moleskine.app.get_current_document()
        self.lang_name.set_text(doc.language.name)
        
        self.rule_glob.connect('clicked', self.set_rule_type, 0)
        self.rule_name_re.connect('clicked', self.set_rule_type, 1)
        self.rule_text_re.connect('clicked', self.set_rule_type, 2)
    
    def setup_editor_tab(self, lang):
        self.tab_size.set_value(lang.tab_size)
        self.soft_tabs.set_active(lang.soft_tabs)
        self.show_indent.set_active(lang.show_indent)
        self.indent_size.set_value(lang.indent_size)
        self.auto_indent.set_active(lang.auto_indent)
        if lang.edge_indicator == 2:
            self.edge_background.set_active(1)
        elif lang.edge_indicator == 1:
            self.edge_line.set_active(1)
        else:
            self.edge_none.set_active(1)
        self.edge_column.set_value(lang.edge_column)
        (r, g, b) = _parse_color(lang.edge_color)
        self.edge_color.set_i8(r, g, b, 0xFF)
    
    def setup_highlight_tab(self, lang):
        for l in Languages.Lexers.keys():
            if Languages.Lexers[l] == lang.lexer:
                lexer_name = l
                break
        self.lexer.set_text(lexer_name)
    
    def setup_keywords_tab(self, lang):
        print lang.api_file
        if lang.api_file is not None:
            self.api_file.gtk_entry().set_text(lang.api_file)
        for kwclass in range(5):
            if lang.keywords[kwclass] is not None:
                kwlist = self['keywords%i_list' % (kwclass + 1)]
                kwlist.freeze()
                kwlist.delete_text(0, -1)
                kwlist.insert_defaults(lang.keywords[kwclass])
                kwlist.thaw()
    
    def setup_ftypes_tab(self, lang):
        self.rules_list.clear()
        self.rules_list.freeze()
        for rule in lang.rules:
            if rule[0] == 0:
                rule_type = _('Shell glob')
            elif rule[0] == 1:
                rule_type = ('Filename regexp')
            else:
                rule_type = _('Text regexp')
            row = self.rules_list.append((rule_type, rule[1]))
            self.rules_list.set_row_data(row, rule[0])
        self.rules_list.thaw()
    
    def language_changed(self, w, user_data=None):
        lang_name = self.lang_name.get_text()
        if not Moleskine.app.langs.has_key(lang_name):
            return
        lang = Moleskine.app.langs[lang_name]
        
        self.setup_editor_tab(lang)
        self.setup_highlight_tab(lang)
        self.setup_keywords_tab(lang)
        self.setup_ftypes_tab(lang)
    
    def lexer_changed(self, w, data=None):
        from copy import deepcopy
        
        lang_name = self.lang_name.get_text()
        if not Moleskine.app.langs.has_key(lang_name):
            return
        lang = Moleskine.app.langs[lang_name]
        lexer_name = self.lexer.get_text()
        lexer = Languages.Lexers[lexer_name]
        
        self.styles_list.unselect_all()
        self.styles_list.clear()
        self.styles_list.freeze()
        if Languages.Styles.has_key(lexer):
            styles = Languages.Styles[lexer]
            for s in styles.keys():
                row = self.styles_list.append((s,))
                if styles.has_key(s) and lang.styles.has_key(styles[s]):
                    self.styles_list.set_row_data(row,
                            (styles[s], deepcopy(lang.styles[styles[s]])))
        self.styles_list.thaw()
    
    def style_selected(self, clist, row, column, event, user_data=None):
        from Languages import _merge_font
        
        style = clist.get_row_data(row)
        if style is None:
            style = {'font': None,
                     'fore': None,
                     'back': None }
        else:
            style = style[1]
        
        if style['font'] is not None:
            font = _merge_font(style['font'])
        else:
            font = Moleskine.app.prefs['default style/font']
        self.style_font.set_font_name(font)
        if style['fore'] is not None:
            r, g, b = _parse_color(int(style['fore']))
        else:
            r, g, b = _parse_color(int(
                    Moleskine.app.prefs['default style/fore']))
        self.style_fore.set_i8(r, g, b, 0xFF)
        if style['back'] is not None:
            r, g, b = _parse_color(int(style['back']))
        else:
            r, g, b = _parse_color(int(
                    Moleskine.app.prefs['default style/back']))
        self.style_back.set_i8(r, g, b, 0xFF)
        
        self.style_font.set_sensitive(1)
        self.style_fore.set_sensitive(1)
        self.style_back.set_sensitive(1)
        self.style_reset.set_sensitive(1)
    
    def style_unselected(self, clist, row, column, event, user_data=None):
        from Languages import _split_font
        style = clist.get_row_data(row)
        if style is None:
            style = {'font': None,
                     'fore': None,
                     'back': None }
        else:
            style = style[1]
        
        style['font'] = _split_font(self.style_font.get_font_name())
        style['fore'] = str(_parse_color(self.style_fore.get_i8()))
        style['back'] = str(_parse_color(self.style_back.get_i8()))
        
        self.style_font.set_sensitive(0)
        self.style_fore.set_sensitive(0)
        self.style_back.set_sensitive(0)
        self.style_reset.set_sensitive(0)
    
    def reset_style_clicked(self, w, user_data=None):
        row = self.styles_list.selection[0]
        style = self.styles_list.get_row_data(row)
        style[1]['font'] = None
        style[1]['fore'] = None
        style[1]['back'] = None
        self.styles_list.select_row(row, 0)
    
    def new_rule_clicked(self, w, user_data=None):
        row = self.rules_list.append((_('Shell glob'), ''))
        self.rules_list.set_row_data(row, 0)
        self.rules_list.select_row(row, 0)
        self.rule_entry.set_text('')
        self.rule_entry.grab_focus()
    
    def del_rule_clicked(self, w, user_data=None):
        self.rules_list.remove(self.rules_list.selection[0])
    
    def edit_rule(self, w, user_data=None):
        if len(self.rules_list.selection) > 0:
            row = self.rules_list.selection[0]
            self.rules_list.set_text(row, 1, w.get_text())
    
    def set_rule_type(self, w, user_data=None):
        if w.get_active() == 1:
            row = self.rules_list.selection[0]
            if user_data == 0:
                self.rules_list.set_text(row, 0, _('Shell glob'))
            elif user_data == 1:
                self.rules_list.set_text(row, 0, _('Filename regexp'))
            else:
                self.rules_list.set_text(row, 0, _('Text regexp'))
    
    def rule_selected(self, clist, row, column, event, user_data=None):
        rule = clist.get_text(row, 1)
        self.rule_entry.set_text(rule)
        rule_type = clist.get_row_data(row)
        if rule_type == 0:
            self.rule_glob.set_active(1)
        elif  rule_type == 1:
            self.rule_name_re.set_active(1)
        else:
            self.rule_text_re.set_active(1)
        
        self.rule_entry.set_sensitive(1)
        self.del_rule.set_sensitive(1)
        self.rule_glob.set_sensitive(1)
        self.rule_name_re.set_sensitive(1)
        self.rule_text_re.set_sensitive(1)
    
    def rule_unselected(self, clist, row, column, event, user_data=None):
        self.rule_entry.set_text('')
        self.rule_glob.set_active(1)
        
        self.rule_entry.set_sensitive(0)
        self.del_rule.set_sensitive(0)
        self.rule_glob.set_sensitive(0)
        self.rule_name_re.set_sensitive(0)
        self.rule_text_re.set_sensitive(0)
    
    def on_lang_name_activate(self, w, user_data=None):
        w.set_editable(0)
        name = w.get_text()
        if not Moleskine.app.langs.has_key(name):
            Moleskine.app.langs[name] = \
                    Languages.Language(name, GTKSCINTILLA.LEXER_NULL)
            lang_names = Moleskine.app.langs.keys()
            lang_names.sort()
            self.lang_name_combo.set_popdown_strings(lang_names)
            w.set_text(name)
    
    def new_language(self, w, user_data=None):
        self.lang_name.set_text('')
        self.lang_name.set_editable(1)
        self.lang_name.grab_focus()
    
    def copy_language(self, w, user_data=None):
        pass
    
    def del_language(self, w, user_data=None):
        if self.lang_name.get_text() == 'Plain text':
            gnome.ui.GnomeMessageBox(
                    _('You can\'t remove this language.'),
                    gnome.uiconsts.MESSAGE_BOX_ERROR,
                    gnome.uiconsts.STOCK_BUTTON_OK).run_and_close()
            return
        del(Moleskine.app.langs[self.lang_name.get_text()])
        lang_names = Moleskine.app.langs.keys()
        lang_names.sort()
        self.lang_name_combo.set_popdown_strings(lang_names)
        self.lang_name.set_text('Plain text')
    
    def apply(self):
        from copy import deepcopy
        
        langs = Moleskine.app.langs
        lang_name = self.lang_name.get_text()
        lang = langs[lang_name]
        
        lang.tab_size = self.tab_size.get_value_as_int()
        lang.soft_tabs = self.soft_tabs.get_active()
        lang.indent_size = self.indent_size.get_value_as_int()
        lang.show_indent = self.show_indent.get_active()
        lang.auto_indent = self.auto_indent.get_active()
        if self.edge_none.get_active():
            lang.edge_indicator = 0
        elif self.edge_line.get_active():
            lang.edge_indicator = 1
        else:
            lang.edge_indicator = 2
        lang.edge_column = self.edge_column.get_value_as_int()
        lang.edge_color = _parse_color(self.edge_color.get_i8())
        
        lang.api_file = self.api_file.get_full_path(0)
        
        if len(self.styles_list.selection) > 0:
            self.styles_list.unselect_all()
        lang.lexer = Languages.Lexers[self.lexer.get_text()]
        for row in range(self.styles_list.rows):
            style = self.styles_list.get_row_data(row)
            if style is None:
                continue
            lang.styles[style[0]] = deepcopy(style[1])
        
        lang.keywords = [None, None, None, None, None]
        lang.keywords[0] = self.keywords1_list.get_chars(0, -1)
        lang.keywords[1] = self.keywords2_list.get_chars(0, -1)
        lang.keywords[2] = self.keywords3_list.get_chars(0, -1)
        lang.keywords[3] = self.keywords4_list.get_chars(0, -1)
        lang.keywords[4] = self.keywords5_list.get_chars(0, -1)
        
        lang.rules = []
        for row in range(self.rules_list.rows):
            type = self.rules_list.get_row_data(row)
            rule = self.rules_list.get_text(row, 1)
            lang.rules.append((type, rule))
        
        Moleskine.app.langs.save()
        for doc in Moleskine.app.documents:
            doc.reload_language()
    
    def destroy(self, w, user_data=None):
        global langs_dialog
        langs_dialog = None
    
    def clicked(self, w, button, user_data=None):
        if button == 0:
            self.apply()
            self.close()
        elif button == 1:
            self.apply()
        else:
            self.close()

class SearchDialog(GladeDialog):
    def __init__(self):
        GladeDialog.__init__(self, 'search_dialog')
        self.set_transient_for(Moleskine.app)
        self.editable_enters(self.search_combo.entry)
        self.search_combo.set_case_sensitive(1)
        self.search_combo.set_use_arrows_always(1)
    
    def show(self):
        GladeDialog.show(self)
        doc = Moleskine.app.get_current_document()
        if doc.get_selection_start() != doc.get_selection_end():
            self.search_entry.set_text(doc.get_sel_text())
        else:
            self.search_entry.set_text('')
        self.search_entry.select_region(0, -1)
        self.get_window()._raise()
        self.search_entry.grab_focus()
    
    def get_flags(self):
        from Document import FindFlags
        
        FindFlags.match_case = self.match_case_check.get_active()
        FindFlags.whole_words = self.whole_words_check.get_active()
        FindFlags.word_start = self.word_start_check.get_active()
        FindFlags.regexp = self.regexp_check.get_active()
        FindFlags.backwards = self.backwards_check.get_active()
        FindFlags.text = self.search_entry.get_text()
    
    def regexp_toggled(self, w, user_data=None):
        if w.get_active():
            self.whole_words_check.set_sensitive(0)
            self.word_start_check.set_sensitive(0)
        else:
            self.whole_words_check.set_sensitive(1)
            self.word_start_check.set_sensitive(1)
    
    def search(self, info = 1):
        from Document import FindFlags
        
        self.get_flags()
        doc = Moleskine.app.get_current_document()
        if not info:
            pos = doc.get_current_pos()
            doc.goto_pos(0)
        result = doc.search_text(
            text = FindFlags.text,
            match_case = FindFlags.match_case,
            whole_words = FindFlags.whole_words,
            word_start = FindFlags.word_start,
            regexp = FindFlags.regexp,
            backwards = FindFlags.backwards)
        if result is None and info:
            gnome.ui.GnomeMessageBox(
                    _('No matches for: "%s".') % FindFlags.text,
                    gnome.uiconsts.MESSAGE_BOX_INFO,
                    gnome.uiconsts.STOCK_BUTTON_OK).run_and_close()
        elif result is not None:
            doc.ensure_visible(doc.line_from_position(result[0]))
        if result is None and not info:
            doc.goto_pos(pos)
        return result
    
    def destroy(self, w, user_data=None):
        global find_dialog
        find_dialog = None
    
    def clicked(self, w, button, user_data=None):
        if button == 0:
            self.close()
            self.search()
        else:
            self.close()

class PromptDialog(GladeDialog):
    def __init__(self, text, replace_text):
        self.text = text
        self.replace_text = replace_text
        self.buttonclicked = 0
        GladeDialog.__init__(self, 'replace_prompt_dialog')

    def show(self):
        #GladeDialog.show(self)
        self.replace_label.set_text(_('Replace "%s" with "%s"?') % (self.text, self.replace_text))
        self.get_window()._raise()
        #self.button7.grab_focus()
        self.run()
        return self.buttonclicked   # replace = 0, skip = 1, all = 2, one = 3, cancel = 4

    def clicked(self, w, button, user_data=None):
        self.buttonclicked = button
        self.close()

class ReplaceDialog(SearchDialog):
    def __init__(self):
        GladeDialog.__init__(self, 'replace_dialog')
        self.set_transient_for(Moleskine.app)
        self.editable_enters(self.search_combo.entry)
        self.search_combo.set_case_sensitive(1)
        self.search_combo.set_use_arrows_always(1)
        self.editable_enters(self.replace_combo.entry)
        self.replace_combo.set_case_sensitive(1)
        self.replace_combo.set_use_arrows_always(1)
    
    def show(self):
        SearchDialog.show(self)
        self.replace_entry.set_text('')
    
    def get_flags(self):
        from Document import FindFlags
        
        SearchDialog.get_flags(self)
        FindFlags.replace_prompt = self.replace_prompt_check.get_active()
        FindFlags.replace_all = self.replace_all_radio.get_active()
        FindFlags.replace_selection = self.replace_sel_radio.get_active()
        FindFlags.replace_text = self.replace_entry.get_text()
    
    def build_flags(self):
        from Document import FindFlags
        
        return FindFlags.backwards * GTKSCINTILLA.FIND_DOWN + \
               FindFlags.match_case * GTKSCINTILLA.FIND_MATCH_CASE + \
               FindFlags.whole_words * GTKSCINTILLA.FIND_WHOLE_WORDS + \
               FindFlags.word_start * GTKSCINTILLA.FIND_WORD_START + \
               FindFlags.regexp * GTKSCINTILLA.FIND_REGEXP
    
    def regexp_toggled(self, w, user_data=None):
        if w.get_active():
            self.whole_words_check.set_sensitive(0)
            self.word_start_check.set_sensitive(0)
        else:
            self.whole_words_check.set_sensitive(1)
            self.word_start_check.set_sensitive(1)
    
    def replace_multi(self, all=0):
        from Document import FindFlags
        
        flags = self.build_flags()
        
        doc = Moleskine.app.get_current_document()
        doc.begin_undo_action()
        
        count = 0
        target_start = target_end = 0
        
        if all:
            replace_start = 0
            replace_end = doc.get_length()
        else:
            replace_start = doc.get_selection_start()
            replace_end = doc.get_selection_end()
        
        regexp = FindFlags.regexp
        find_text = FindFlags.text
        replace_text = FindFlags.replace_text
        prompt_on = FindFlags.replace_prompt
        replace_action = 0
        
        result = doc.find_text(flags, find_text, (replace_start, replace_end))
        while result and replace_action < 3:
            target_start, target_end = result
            doc.set_selection_start(target_start)
            doc.set_selection_end(target_end)
            doc.ensure_visible(doc.line_from_position(target_start))
            
            replace_action = 0
            if prompt_on:
                prompt_dialog = PromptDialog(doc.get_sel_text(), replace_text)
                replace_action = prompt_dialog.show()
            if replace_action == 0:      # replace
                pass
            elif replace_action == 1:    # skip
                result = doc.find_text(flags, find_text,
                                       (target_end, replace_end))
                continue
            elif replace_action == 2:    # all
                prompt_on = 0
            elif replace_action == 3:    # one
                pass
            elif replace_action == 4:    # cancel
                break
            
            count = count + 1
            doc.set_target_start(target_start)
            doc.set_target_end(target_end)
            len_replace = doc.replace_target(regexp, replace_text)
            replace_start = target_start + len_replace
            if not all:
                replace_end = replace_end + \
                              (len_replace - (target_end - target_start))
            else:
                replace_end = doc.get_length()
            result = doc.find_text(flags, find_text,
                                   (replace_start, replace_end))
        doc.goto_pos(doc.get_target_end())
        doc.end_undo_action()
        if replace_dialog != 4:
            gnome.ui.GnomeMessageBox(
                        _('Done.\nReplaced %i strings.') % (count, ),
                        gnome.uiconsts.MESSAGE_BOX_INFO,
                        gnome.uiconsts.STOCK_BUTTON_OK).run_and_close()
        return count
    
    def replace(self):
        from Document import FindFlags

        doc = Moleskine.app.get_current_document()
        sel_start = doc.get_selection_start()
        sel_end = doc.get_selection_end()
        
        self.get_flags()
        
        count = 0
        if FindFlags.replace_all:
            count = self.replace_multi(all=1)
        elif FindFlags.replace_selection:
            count = self.replace_multi()
        else:
            self.search()
            search_sel_start = doc.get_selection_start()
            search_sel_end = doc.get_selection_end()
            
            if search_sel_start != search_sel_end:
                count = 1
                prompt_on = FindFlags.replace_prompt
                replace_action = 0
                if prompt_on:
                    prompt_dialog = PromptDialog(doc.get_sel_text(), FindFlags.replace_text)
                    replace_action = prompt_dialog.show()
                if replace_action == 1 or replace_action == 4:      # skip or cancel
                    return
                doc.set_target_start(search_sel_start)
                doc.set_target_end(search_sel_end)
                replace_length = doc.replace_target(FindFlags.regexp,
                                                    FindFlags.replace_text)
        doc.set_selection_start(sel_start)
        doc.set_selection_end(sel_end + count * (len(FindFlags.replace_text) - len(FindFlags.text)))
    
    def destroy(self, w, user_data=None):
        global replace_dialog
        replace_dialog = None
    
    def clicked(self, w, button, user_data=None):
        if button == 0:
        #    self.search()
        #elif button == 1:
            self.close()
            self.replace()
        else:
            self.close()

class SearchInFilesDialog(SearchDialog):
    def __init__(self):
        GladeDialog.__init__(self, 'search_in_files_dialog')
        self.set_transient_for(Moleskine.app)
        self.editable_enters(self.search_combo.entry)
        self.search_combo.set_case_sensitive(1)
        self.search_combo.set_use_arrows_always(1)
        self.editable_enters(self.pattern_combo.entry)
        self.pattern_combo.set_case_sensitive(1)
        self.pattern_combo.set_use_arrows_always(1)
    
    def show(self):
        if self.directory_entry.get_text() == '' and \
           Moleskine.app.get_current_document().filepath is not None:
            self.directory_entry.set_text(os.path.dirname(Moleskine.app.get_current_document().filepath) + '/')
        SearchDialog.show(self)
    
    def get_flags(self):
        from Document import FindFlags
        
        FindFlags.match_case = self.match_case_check.get_active()
        FindFlags.whole_words = self.whole_words_check.get_active()
        FindFlags.word_start = self.word_start_check.get_active()
        FindFlags.regexp = self.regexp_check.get_active()
        FindFlags.backwards = 0
        FindFlags.text = self.search_entry.get_text()
        
        FindFlags.in_open_files = self.open_files_radio.get_active()
        FindFlags.in_directory_files = self.directory_files_radio.get_active()
        FindFlags.recursive_directory = self.recursive_directory_check.get_active()
        FindFlags.in_all_files = self.all_files_radio.get_active()
        FindFlags.in_pattern_files = self.pattern_files_radio.get_active()
        FindFlags.in_files_pattern = self.pattern_entry.get_text()
        FindFlags.in_directory = self.directory_entry.get_text()
        if FindFlags.in_directory == '':
            FindFlags.in_directory = os.path.dirname(Moleskine.app.get_current_document().filepath)
        FindFlags.in_directory = os.path.expanduser(FindFlags.in_directory)
        FindFlags.in_directory = os.path.expandvars(FindFlags.in_directory)
        FindFlags.in_directory = os.path.abspath(FindFlags.in_directory)
    
    def build_flags(self):
        from Document import FindFlags
        
        return FindFlags.match_case * GTKSCINTILLA.FIND_MATCH_CASE + \
               FindFlags.whole_words * GTKSCINTILLA.FIND_WHOLE_WORDS + \
               FindFlags.word_start * GTKSCINTILLA.FIND_WORD_START + \
               FindFlags.regexp * GTKSCINTILLA.FIND_REGEXP
    
    def regexp_toggled(self, w, user_data=None):
        if w.get_active():
            self.whole_words_check.set_sensitive(0)
            self.word_start_check.set_sensitive(0)
        else:
            self.whole_words_check.set_sensitive(1)
            self.word_start_check.set_sensitive(1)
    
    def matches(self, filepath):
        from fnmatch import fnmatch
        from Document import FindFlags
        
        if fnmatch(filepath, FindFlags.in_files_pattern):
            return 1
        return 0
    
    def mark_tab(self, notebook=None, page_num=0, color='#ff0000'):
        if notebook is None:
            return
        tab_label = notebook.get_tab_label(notebook.get_nth_page(page_num))
        tab_label_style = tab_label.get_style().copy()
        label_color = tab_label.get_colormap().alloc(color)
        tab_label_style.fg[GTK.STATE_NORMAL] = label_color
        tab_label.set_style(tab_label_style)
        
    def get_dir_files(self, subdir):
        from Document import FindFlags
        
        dir_files = os.listdir(subdir)
        for j in range(len(dir_files)):
            filepath = subdir + '/' + dir_files[j]
            dir_files[j] = filepath
            if os.path.isdir(filepath):
                if FindFlags.recursive_directory and not os.path.islink(filepath):
                    dir_files = dir_files + self.get_dir_files(filepath)
        return dir_files
    
    def search_in_files(self):
        from Document import FindFlags
        from gtk import GdkColor
        
        documents = Moleskine.app.documents
        notebook = Moleskine.app.notebook
        pos = notebook.get_current_page()
        first_found = None
        times_found = 0
        
        self.get_flags()
        
        Moleskine.app.switch = 0
        if FindFlags.in_directory_files:
            directory_files = self.get_dir_files(FindFlags.in_directory)
            search_directory_files = []
            for i in range(len(directory_files)):
                ok = 1
                filepath = directory_files[i]
                if os.path.isdir(filepath) or not os.path.isfile(filepath):
                    continue
                if FindFlags.in_pattern_files:
                    if not self.matches(filepath):
                      continue
                for doc in Moleskine.app.documents:
                    if filepath == doc.filepath:
                        ok = 0
                        break
                if ok: search_directory_files.append(filepath)
            for file in search_directory_files:
                Moleskine.app.open_document(file)
        
        close = []
        if FindFlags.in_open_files or FindFlags.in_directory_files:
            for i in range(len(documents)):
                notebook.set_page(i)
                label = notebook.get_tab_label(notebook.get_nth_page(i))
                label.set_rc_style()    # Default style
                if FindFlags.in_directory_files:
                    if not FindFlags.in_directory == os.path.dirname(documents[i].filepath):
                        if FindFlags.recursive_directory and string.find(os.path.dirname(documents[i].filepath), FindFlags.in_directory) != 0:
                            continue
                if FindFlags.in_pattern_files:
                    if not self.matches(documents[i].filepath):
                        continue
                result = self.search(0)
                if result is not None:
                    if first_found is None:
                        first_found = i
                    times_found = times_found + 1
                    # Mark tab label
                    if FindFlags.in_directory_files and documents[i].filepath in search_directory_files:
                        self.mark_tab(notebook, i, 'blue')
                    else:
                        self.mark_tab(notebook, i, 'red')
                else:
                    if FindFlags.in_directory_files and documents[i].filepath in search_directory_files:
                        close.append(i)
        for i in range(len(close)):
            notebook.set_page(close[i] - i)
            Moleskine.app.close_document()
        Moleskine.app.switch = 1
        if first_found is None:
            notebook.set_page(pos)
        else:
            notebook.set_page(first_found)
        if times_found > 0:
            gnome.ui.GnomeMessageBox(
                    _('Done.\nString found in %i files.') % (times_found, ),
                    gnome.uiconsts.MESSAGE_BOX_INFO,
                    gnome.uiconsts.STOCK_BUTTON_OK).run_and_close()
        else:
            gnome.ui.GnomeMessageBox(
                    _('String not found in any files.'),
                    gnome.uiconsts.MESSAGE_BOX_INFO,
                    gnome.uiconsts.STOCK_BUTTON_OK).run_and_close()

    
    def destroy(self, w, user_data=None):
        global search_in_files_dialog
        search_in_files_dialog = None
    
    def clicked(self, w, button, user_data=None):
        if button == 0:
            self.close()
            self.search_in_files()
        else:
            self.close()

    def key_press_event(self, w, event):
        if event.keyval == GDK.Return:
            self.close()
            self.search_in_files()

def options():
    global options_dialog
    
    if options_dialog is None:
        options_dialog = OptionsDialog()
    else:
        options_dialog.get_window()._raise()

def languages():
    global langs_dialog
    
    if langs_dialog is None:
        langs_dialog = LanguagesDialog()
    else:
        langs_dialog.get_window()._raise()

def search():
    global search_dialog
    
    if search_dialog is None:
        search_dialog = SearchDialog()
    search_dialog.show()

def search_in_files():
    global search_in_files_dialog
    
    if search_in_files_dialog is None:
        search_in_files_dialog = SearchInFilesDialog()
    search_in_files_dialog.show()

def replace():
    global replace_dialog
    
    if replace_dialog is None:
        replace_dialog = ReplaceDialog()
    replace_dialog.show()



# MoleskineApp.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2000 - 2002   Michele Campeotto <micampe@micampe.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import os.path
import stat
import popen2

import gtk
import GTK
import GDK
import gnome.ui
import gnome.uiconsts

from Document import Document
from Config import Config
from Languages import LanguagesList
import menu
import toolbar

TARGET_STRING = 0
TARGET_ROOTWIN = 1
target = [
    ('STRING', 0, TARGET_STRING),
    ('text/plain', 0, TARGET_STRING),
    ('application/x-rootwin-drop', 0, TARGET_ROOTWIN)]

in_event = 0

class MoleskineApp(gnome.ui.GnomeApp):
    def __init__(self):
        gnome.ui.GnomeApp.__init__(self, 'GnomeMoleskine', 'Moleskine')
        
        self.switch = 1
        self.documents = []
        self.reload_notification = 0
        self.last_page = [0, 0]
        self.load_prefs()
        
        self.notebook = gtk.GtkNotebook()
        self.notebook.connect_after('switch-page', self.__switch_page)
        self.notebook.connect_after('set_focus_child', self.__focus_notebook_tab)
        self.notebook.set_scrollable(1)
        self.notebook.drag_dest_set(gtk.DEST_DEFAULT_ALL, target, GDK.ACTION_COPY | GDK.ACTION_MOVE)
        self.notebook.connect('drag_drop',self.drag_drop)
        self.notebook.connect('drag_data_received',self.drag_data_received)
        self.notebook.show()
        self.set_contents(self.notebook)
        
        self.status = gnome.ui.GnomeAppBar()
        self.progress = self.status.get_progress()
        self.set_statusbar(self.status)
        
        self.set_wmclass('main_window', 'Moleskine')
        self.set_geometry_hints(min_width=0, min_height=0)
        self.connect('key_press_event', self.key_press)
        if int(self.prefs['keyboard/keybindings']) == 2:
            import Emacs
            self.create_menus(Emacs.main_menu)
            self.Emacs_key_press = Emacs.key_press
        elif int(self.prefs['keyboard/keybindings']) == 1:
            import GnomeMenu
            self.create_menus(GnomeMenu.main_menu)
        else:
            self.create_menus(menu.main_menu)
        self.create_toolbar(toolbar.main_toolbar)
        self.connect('delete_event', menu.exit)
        self.drag_dest_set(gtk.DEST_DEFAULT_ALL, target, GDK.ACTION_COPY | GDK.ACTION_MOVE)
        self.connect('drag_drop',self.drag_drop)
        self.connect('drag_data_received',self.drag_data_received)
        self.connect_after('focus_in_event', self.__focus_notebook_tab)
        self.show_all()
        
        self.load_langs()
        self.load_mru()
        self.load_rsess()
    
    def key_press(self, w, event):
        """Handle keypresses."""
        if event.state & GDK.MOD1_MASK and not event.state & (GDK.SHIFT_MASK | GDK.CONTROL_MASK):
            if event.keyval >= GDK._1 and event.keyval <= GDK._9:
                self.notebook.set_page(event.keyval - GDK._1)
            elif event.keyval == GDK._0:
                self.notebook.set_page(9)
            elif int(self.prefs['keyboard/keybindings']) == 2:
            #forward alt keypresses to emacs keybindings if we don't use them
                self.Emacs_key_press(w, event)
        else:
            if int(self.prefs['keyboard/keybindings']) == 2:
                self.Emacs_key_press(w, event)

    def drag_drop(self, w, context, x, y, time):
        w.drag_get_data(context, context.targets[0], time)
    
    def drag_data_received(self, w, context, x, y, data, info, time):
        self.open_document(data.data)
    
    def new_document(self):
        """Create a new document."""
        doc = Document()
        self.documents.append(doc)
        self.notebook.append_page(doc, doc.label)
        self.notebook.set_page(-1)
        self.update_title()
    
    def open_document(self, filepath):
        """Open a document.
        
        The file name can contain relative paths, special shell chars (~) and
        environment variables.
        If the current document is empty the newly created one will replace it.
        If the given file doesn't exist you will be asked about to create it.
        """
        from string import strip
        import urlparse
        url = urlparse.urlparse(filepath)
        filepath = strip(url[2])
        
        # remove null bytes from GMC filenames.
        from string import replace
        filepath = replace(filepath, "\0","")
        filepath = strip(filepath)
        
        filepath = os.path.expanduser(filepath)
        filepath = os.path.expandvars(filepath)
        filepath = os.path.abspath(filepath)
        
        if os.path.isdir(filepath):
            gnome.ui.GnomeMessageBox(
                _('Cannot open `%s\': is a directory.') % \
                os.path.basename(filepath),
                gnome.uiconsts.MESSAGE_BOX_ERROR,
                gnome.uiconsts.STOCK_BUTTON_OK).run_and_close()
        elif not os.path.exists(filepath):
            res = gnome.ui.GnomeMessageBox(
                    _('File `%s\' not found.\nDo you want to create it?') % \
                            os.path.basename(filepath),
                    gnome.uiconsts.MESSAGE_BOX_QUESTION,
                    gnome.uiconsts.STOCK_BUTTON_YES,
                    gnome.uiconsts.STOCK_BUTTON_NO).run_and_close()
            if res == 0:
                self.__load(filepath)
                self.get_current_document().set_text(' ')
                self.get_current_document().set_text('')
            else:
                if len(self.documents) == 0:
                    self.new_document()
        elif not os.access(filepath, os.R_OK):
            gnome.ui.GnomeMessageBox(
                _('Cannot open `%s\': permission denied.') % \
                os.path.basename(filepath),
                gnome.uiconsts.MESSAGE_BOX_ERROR,
                gnome.uiconsts.STOCK_BUTTON_OK).run_and_close()
            return
        else:
            self.__load(filepath)
        self.get_current_document().grab_focus()
        self.update_title()
    
    def __load(self, filepath):
        doc = self.get_current_document()
        
        for i in range(len(self.documents)):
            if self.documents[i].filepath == filepath:
                self.notebook.set_page(i)
                if not self.documents[i].is_modified():
                    self.documents[i].revert()
                return
        if doc is None or not doc.is_empty():
            self.new_document()
            doc = self.get_current_document()
        doc.load_file(filepath)
        self.add_to_mru(filepath)
    
    def save_document(self, filepath = None):
        self.get_current_document().save_file(filepath)
    
    def close_document(self):
        """Close the active document.
        
        If the document to be closed is the last one, a new empy document will
        be created.
        If the document has been modified and not saved you will be asked if
        you want to save the changes.
        """
        def save_and_close(fs, user_data = None):
            import Moleskine
            
            app = Moleskine.app
            app.save_document(fs.get_filename())
            app.close_document()
        
        doc = self.get_current_document()
        if not doc.can_close():
            res = gnome.ui.GnomeMessageBox(
                    _('File `%s\' has been modified\nand not saved.' +
                    ' Do you want to save it now?') % doc.filename,
                    gnome.uiconsts.MESSAGE_BOX_QUESTION,
                    gnome.uiconsts.STOCK_BUTTON_YES, 
                    gnome.uiconsts.STOCK_BUTTON_NO,
                    gnome.uiconsts.STOCK_BUTTON_CANCEL).run_and_close()
            if res == 0:
                if doc.filepath is not None:
                    doc.save_file()
                else:
                    self.ask_file('save', save_and_close)
                    return
            elif res == 2 or res < 0:
                return
        page = self.notebook.get_current_page()
        if self.last_page[1] is None:
            self.last_page[1] = page
        elif self.last_page[1] > page:
            self.last_page[1] = self.last_page[1] - 1
        self.last_page[0] = self.last_page[1]
        switch = self.switch
        self.switch = 0
        self.notebook.remove_page(page)
        self.documents.remove(doc)
        self.switch = switch
        if len(self.documents) == 0:
            self.new_document()
        else:
            if self.notebook.get_current_page() == self.last_page[1]:
                force_update = 1
            else:
                force_update = 0
            self.notebook.set_page(self.last_page[1])
            if force_update == 1:
                self.update_title()
            for i in range(len(self.documents)):
                if self.documents[i].is_modified():
                    asterisk = '*'
                else:
                    asterisk = ''
                if self.documents[i].filename is not None:
                    self.documents[i].label.set_text('%s%i. %s' % (asterisk, i + 1, self.documents[i].filename))
                else:
                    self.documents[i].label.set_text('%s%i. %s' % (asterisk, i + 1, _('Unnamed')))
            self.get_current_document().grab_focus()
    
    def ask_file(self, type, ok_cb):
        if type == 'load':
            title = _('Open file...')
            wm_name = 'open_dialog'
        elif type == 'save':
            title = _('Save as...')
            wm_name = 'save_dialog'
        elif type == 'load_session':
            title = _('Open session...')
            wm_name = 'open_dialog'
        elif type == 'save_session':
            title = _('Save session as...')
            wm_name = 'save_dialog'
        else:
            title = type
            wm_name = 'file_dialog'
        
        fs = gtk.GtkFileSelection(title)
        #fs.set_wmclass(wm_name, 'Moleskine')
        fs.set_transient_for(self)
        if type[:4] != 'save':
            fs.file_list.set_selection_mode(GTK.SELECTION_MULTIPLE)
        
        doc = self.get_current_document()
        if doc is not None and doc.filepath is not None:
            startdir = os.path.dirname(doc.filepath)
            fs.set_filename(startdir + '/')
        elif self.last_page[1] is not None and \
             self.documents[self.last_page[1]] is not None and \
             self.documents[self.last_page[1]].filepath is not None:
            startdir = os.path.dirname(self.documents[self.last_page[1]].filepath)
            fs.set_filename(startdir + '/')
        if ok_cb is not None:
            fs.ok_button.connect_object('clicked', ok_cb, fs)
        
        fs.ok_button.connect_object('clicked', gtk.GtkWidget.destroy, fs)
        fs.cancel_button.connect_object('clicked', gtk.GtkWidget.destroy, fs)
        fs.show()
    
    def get_current_document(self):
        """Returns the currently active document.
        
        If there are no documents (should never happen) it will return None.
        """
        if len(self.documents) > 0:
            return self.documents[self.notebook.get_current_page()]
        else:
            return None
    
    def update_ru_menu(self, callback, menu_label, list, sort = 0):
        """ Update recently used items menu
        """
        import string
        
        fix_underscore = lambda label, replace=string.replace: \
                replace(label, '_', '__')
        
        build_item = lambda label, callback=callback: \
                (gnome.uiconsts.APP_UI_ITEM, label, None,
                 callback, None,
                 gnome.uiconsts.APP_PIXMAP_NONE, None, 0, 0)
        
        if int(sort):
            list.sort()
        list = map(fix_underscore, list)
        ru_menu = map(build_item, list)
        self.remove_menus(menu_label, 55)
        self.insert_menus(menu_label, ru_menu)
    
    def add_to_ru(self, list, filepath, size):
        if filepath in list:
            list.remove(filepath)
        list.insert(0, filepath)
        if len(list) > int(size):
            list = list[:size]
    
    def clean_up_ru(self, list):
        """Remove non-existent files from the recently used items list"""
        from os.path import exists
        
        count = 0
        start_val = len(list)
        while count < len(list):
            if not exists(list[count]):
                del(list[count])
                count = count - 1     #step back into the changed list
            count = count + 1
        
        #print start_val - len(list), " files removed"
    
    def load_mru(self):
        self.mru = Config('Moleskine/PreviousFiles')
        self.files = []
        count = self.mru['files/count']
        if count is not None:
            for i in range(int(count)):
                self.files.append(self.mru['files/file_%i' % i])
            self.update_mru_menu()
    
    def save_mru(self):
        self.files = self.files[:int(self.prefs['history/mru_size'])]
        for i in range(len(self.files)):
            self.mru['files/file_%i' % i] = self.files[i]
        self.mru['files/count'] = len(self.files)
        self.mru.save()
    
    def update_mru_menu(self):
        import copy
        
        temp_list = copy.copy(self.files)[:int(self.prefs['history/mru_size'])]
        self.update_ru_menu(menu.open_recent, menu.MENU_MRU, temp_list, int(self.prefs['history/mru_sort']))
    
    def add_to_mru(self, filepath):
        self.add_to_ru(self.files, filepath, int(self.prefs['history/mru_size']))
        self.update_mru_menu()
    
    def clean_up_mru(self):
        self.clean_up_ru(self.files)
        self.update_mru_menu()
    
    def delete_mru(self):
        self.files = []
        self.update_mru_menu()
    
    def load_rsess(self):
        self.rsess = Config('Moleskine/RecentSessions')
        self.rsessions = []
        count = self.rsess['sessions/count']
        if count is not None:
            for i in range(int(count)):
                self.rsessions.append(self.rsess['sessions/session_%i' % i])
            self.update_rsess_menu()
    
    def save_rsess(self):
        self.rsessions = self.rsessions[:10]
        for i in range(len(self.rsessions)):
            self.rsess['sessions/session_%i' % i] = self.rsessions[i]
        self.rsess['sessions/count'] = len(self.rsessions)
        self.rsess.save()
    
    def update_rsess_menu(self):
        import copy
        
        temp_list = copy.copy(self.rsessions)[:10]
        self.update_ru_menu(menu.open_recent_session, menu.MENU_RSESS, temp_list)
    
    def add_to_rsess(self, filepath):
        self.add_to_ru(self.rsessions, filepath, 10)
        self.update_rsess_menu()
    
    def clean_up_rsess(self):
        self.clean_up_ru(self.rsessions)
        self.update_rsess_menu()
    
    def delete_rsess(self):
        self.rsessions = []
        self.update_rsess_menu()
    
    def load_session(self, filename = None):
        if filename is not None:
            self.session = Config('=' + filename + '=')
            self.add_to_rsess(filename)
        else:
            self.session = Config('Moleskine/Session')
        count = self.session['files/count']
        if count is not None and int(count) > 0:
            if filename is not None:
                self.notebook.set_page(0)
                for i in range(len(self.documents)):
                    self.close_document()
            self.switch = 0
            document_index = 0
            for i in range(int(count)):
                if self.session['files/file_%i' % i] == 'None':
                    self.new_document()
                    document_index = document_index + 1
                else:
		    if self.session['files/file_%i' % i] is not None:
                        self.open_document(self.session['files/file_%i' % i])
	                if (len(self.documents) - document_index) == 1:
	                    doc = self.documents[document_index]
		            if self.session['langs/lang_%i' % i] in self.langs.keys():
		                doc.set_language(self.langs[self.session['langs/lang_%i' % i]])
			    if self.session['pos/pos_%i' % i] is not None:
			        doc.goto_pos(int(self.session['pos/pos_%i' % i]))
                            document_index = document_index + 1
            self.switch = 1
            if not document_index:
                self.new_document()
            self.notebook.set_page(int(self.session['notebook/tab']))
            self.update_title()
    
    def save_session(self, filename = None):
        if int(self.prefs['history/save_session']) or filename is not None:
            if filename is not None:
                self.session = Config('=' + filename + '=')
                self.add_to_rsess(filename)
            else:
                self.session = Config('Moleskine/Session')
            self.session['notebook/tab'] = self.notebook.get_current_page()
            self.session['files/count'] = len(self.documents)
            for i in range(len(self.documents)):
                self.session['files/file_%i' % i] = self.documents[i].filepath
                self.session['langs/lang_%i' % i] = self.documents[i].language.name
                self.session['pos/pos_%i' % i] = self.documents[i].get_current_pos()
            self.session.save()
        else:
            self.session = Config('Moleskine/Session')
            self.session.data.clear()
            self.session['files/count'] = 0
            self.session.save()
    
    def load_prefs(self):
        self.prefs = Config('Moleskine/Preferences')
        
        self.prefs.dirnames = []
        count = int(self.prefs['directory names/count'])
        for n in range(count):
            name = self.prefs['directory names/name_%i' % n]
            directory = self.prefs['directory names/directory_%i' % n]
            
            directory = os.path.expanduser(directory)
            directory = os.path.expandvars(directory)
            directory = os.path.abspath(directory)
            
            self.prefs.dirnames.append((name, directory))
        
        w = int(self.prefs['geometry/width'])
        h = int(self.prefs['geometry/height'])
        self.set_default_size(w, h)
    
    def save_prefs(self):
        count = len(self.prefs.dirnames)
        self.prefs['directory names/count'] = count
        for n in range(count):
            self.prefs['directory names/name_%i' % n] = \
                    self.prefs.dirnames[n][0]
            self.prefs['directory names/directory_%i' % n] = \
                    self.prefs.dirnames[n][1]
        
        if int(self.prefs['geometry/save_win_size']):
            self.prefs['geometry/width'] = self.get_window().width
            self.prefs['geometry/height'] = self.get_window().height
    
    # FIXME: Can't get the right window position! :( This gives
    # the position of the client window (without decorations)
    # and set_uposition places the window with deocorations. Hints?
    #    if int(self.prefs['geometry/save_win_pos']):
    #        self.prefs['geometry/pos_x'] = self.get_window().x
    #        self.prefs['geometry/pos_y'] = self.get_window().y
        self.prefs.save()
    
    def load_langs(self):
        self.langs = LanguagesList()
        self.langs.load()
        
        langs_names = self.langs.keys()
        langs_names.sort()
        build_item = lambda label: \
                (gnome.uiconsts.APP_UI_ITEM, label, None,
                 menu.set_lang, None,
                 gnome.uiconsts.APP_PIXMAP_NONE, None, 0, 0)
        langs_menu = [gnome.uiconsts.UIINFO_RADIOLIST(
                          map(build_item, langs_names))]
        self.remove_menus(menu.MENU_SETTINGS + '/' +
                          menu.MENU_CHANGE_LANGUAGE + '/', len(langs_names))
        self.insert_menus(menu.MENU_SETTINGS+'/' +
                          menu.MENU_CHANGE_LANGUAGE + '/', langs_menu)
    
    def update_styles(self):
        for doc in self.documents:
            doc.setup_default_style()
            doc.setup_caret_style()
            doc.setup_line_numbers_style()
            doc.reload_language()
    
    def update_title(self):
        if not self.switch:
            return
        doc = self.get_current_document()
        if doc.is_modified():
            sep = ' * '
        else:
            sep = ' - '
        self.set_title('Moleskine' + sep + doc.title)
    
    def __switch_page(self, notebook, notebook_page, page_num, data=None):
        if not self.switch:
            return
        doc = self.get_current_document()
        self.reload_notification = 0
        self.last_page[1] = self.last_page[0]
        self.last_page[0] = page_num
        label = notebook.get_tab_label(notebook.get_nth_page(page_num))
        label.set_rc_style()
        self.update_title()
        try:
            langs_names = self.langs.keys()
            langs_names.sort()
            index = langs_names.index(doc.language.name)
            menu_items = self.langs_menu.children()
            if len(langs_names) < len(menu_items):
                menu_items = menu_items[1:]
            menu_items[index].set_active(1)
        except AttributeError:
            pass
        doc.set_view_ws(int(self.prefs['view/ws']))
        doc.set_view_eol(int(self.prefs['view/eol']))
        doc.ensure_visible(doc.line_from_position(doc.get_current_pos()))
        doc.grab_focus()
    
    def __focus_notebook_tab(self, widget, event=None, data=None):
        global in_event
        if event is None:
            return
        #print "widget =>", widget, "event =>", event, "data =>", data
        #return
        doc = self.get_current_document()
        if doc is None:
            return
        # Compare modification times
        #print "file =>", doc.filepath
        if not self.reload_notification and \
           doc.filepath is not None and \
           not os.access(doc.filepath, os.F_OK):
            # File has been removed
            gnome.ui.GnomeMessageBox(
                    _('File `%s\' has been removed on the disk.') % \
                            os.path.basename(doc.filepath),
                    gnome.uiconsts.MESSAGE_BOX_WARNING,
                    gnome.uiconsts.STOCK_BUTTON_OK).run_and_close()
            self.reload_notification = 1
        elif not self.reload_notification and \
             doc.filepath is not None and \
             doc.mtime < os.stat(doc.filepath)[stat.ST_MTIME]:
            # File on disk has changed
            self.reload_notification = 1
            res = gnome.ui.GnomeMessageBox(
                    _('File `%s\' has changed on the disk.\nDo you want to reload it?') % \
                            os.path.basename(doc.filepath),
                    gnome.uiconsts.MESSAGE_BOX_QUESTION,
                    gnome.uiconsts.STOCK_BUTTON_YES,
                    gnome.uiconsts.STOCK_BUTTON_NO,
                    'diff').run_and_close()
            if res == 0:          # reload
                doc.revert()
            elif res == 2:        # make diff
                # os.popen2 is available only on Python 2
                #(diff_input, diff_output) = os.popen2('diff -u - %s' % doc.filepath)
                (diff_output, diff_input) = popen2.popen2('diff -u - %s' % doc.filepath)
                diff_input.write(doc.get_text())
                diff_input.close()
                diff_text = diff_output.read()
                diff_output.close()
                self.new_document()
                doc = self.get_current_document()
                doc.set_text(diff_text)
                doc.set_language(self.langs['diff output'])
        if not in_event:
            in_event = 1
            doc.grab_focus()



# __init__.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2000 - 2002   Michele Campeotto <micampe@micampe.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


try:
    import gettext
    import commands
    
    domain = 'moleskine'
    
    data_dir = commands.getoutput('gnome-config --datadir')
    locale_dir = data_dir + '/locale'
    
    gettext.bindtextdomain(domain, locale_dir)
    gettext.textdomain(domain)
    
    # here from _gladetext module to direct call GNU gettext
    # so libglade can know where to find the mo file
    try:
        import _gtkscintilla
        _gtkscintilla.bindtextdomain(domain, locale_dir)
        _gtkscintilla.textdomain(domain)
    except:
        pass
except IOError:
    pass
    
import __builtin__
__builtin__.__dict__['_'] = gettext.gettext



# toolbar.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2000 - 2002  Michele Campeotto <micampe@micampe.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from gnome.ui import *

import menu

main_toolbar = [
    UIINFO_ITEM_STOCK(_('New'), _('Create a new file'),
            menu.new_document, STOCK_PIXMAP_NEW),
    UIINFO_ITEM_STOCK(_('Open'), _('Load a file from disk'),
            menu.open_document, STOCK_PIXMAP_OPEN),
    UIINFO_ITEM_STOCK(_('Save'), _('Save current file to disk'),
            menu.save_document, STOCK_PIXMAP_SAVE),
    UIINFO_ITEM_STOCK(_('Close'), _('Close current file'),
            menu.close_document, STOCK_PIXMAP_CLOSE),
    UIINFO_SEPARATOR,
    UIINFO_ITEM_STOCK(_('Undo'), _('Undo last action'),
            menu.undo, STOCK_PIXMAP_UNDO),
    UIINFO_ITEM_STOCK(_('Redo'), _('Redo last action'),
            menu.redo, STOCK_PIXMAP_REDO),
    UIINFO_SEPARATOR,
    UIINFO_ITEM_STOCK(_('Cut'), _('Cut selected text to the clipboard'),
            menu.cut, STOCK_PIXMAP_CUT),
    UIINFO_ITEM_STOCK(_('Copy'), _('Copy selected text to the clipboard'),
            menu.copy, STOCK_PIXMAP_COPY),
    UIINFO_ITEM_STOCK(_('Paste'), _('Paste text from the clipboard'),
            menu.paste, STOCK_PIXMAP_PASTE),
    UIINFO_SEPARATOR,
    UIINFO_ITEM_STOCK(_('Search'), _('Search a string in current file'),
            menu.search, STOCK_PIXMAP_SEARCH),
    UIINFO_ITEM_STOCK(_('Replace'), _('Search and replace a string'),
            menu.replace, STOCK_PIXMAP_SRCHRPL),
    UIINFO_SEPARATOR,
    UIINFO_ITEM_STOCK(_('Quit'), _('Quit Moleskine'),
            menu.exit, STOCK_PIXMAP_EXIT)
]



# Emacs.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2000 - 2002   Michele Campeotto <micampe@micampe.it>
#
# Emacs keybindings for Moleskine by Matthew Pratt <mattpratt@yahoo.com>
# Please mail me if I haven't done your favourite emacs keystroke yet
#
# Copyright (c) 2001   Matthew Pratt <mattpratt@yahoo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# [Ade]
# Reimplemented the data structure once again. This time there should be no
# need for further tweaking as it is capable of dealing with all the emacs
# commands I plan to implement support for.
# Cleared all scintilla keybindings.
#Slight problem with escape key inserting spurious characters. Defined an empty
#function so that escape currently does nothing.
#Ideally we would just re-enable the scintilla binding for this key.

# Added:
# The really useful emacs commands for directional movement.
# Go to start of line, end of line, next word, etc
# Alt-/ for auto-complete, insert/overwrite,
#  shifted selections

# TODO
# It might be nice if we could switch bindings/menu commands without restarting

import GDK

import Moleskine
import menu

last_values = (None, None, None, None)

#modifier constants
CTRL="CTRL"
ALT="ALT"
SHIFT="SHIFT"

#functions have to be defined before they can be used in the dictionary

def kill_line(w):
    """emacs style kill-line, cut from position to end of line
    but if there's nothing to cut then cut the line-ending"""
    doc = Moleskine.app.get_current_document()
    pos = doc.get_current_pos()
    line = doc.line_from_position(pos)
    doc.set_sel(pos, doc.get_line_end_position(line))
    
    if (doc.get_sel_text()==''):
    #we're at the end of the line so treat it differently
        c1 = doc.get_char_at(pos)
        c2 = doc.get_char_at(pos+1)
        if c1==13 and c2==10:
        #test for CR+LF combination
        #For some reason I don't understand only numbers work
            doc.set_sel(pos, pos+2)
        else:
            doc.set_sel(pos, pos+1)
        doc.cut()
    else:
        doc.copy() #copy used as cut kills the line-ending as well
        doc.replace_sel('')
    return

def select_all(w):
    doc = Moleskine.app.get_current_document()
    doc.select_all()

def line_up(w):
    doc = Moleskine.app.get_current_document()
    doc.line_up()

def line_down(w):
    doc = Moleskine.app.get_current_document()
    doc.line_down()

def char_left(w):
    doc = Moleskine.app.get_current_document()
    doc.char_left()

def char_right(w):
    doc = Moleskine.app.get_current_document()
    doc.char_right()

def line_start(w):
    doc = Moleskine.app.get_current_document()
    doc.home()

def line_end(w):
    doc = Moleskine.app.get_current_document()
    doc.line_end()

def document_start(w):
    doc = Moleskine.app.get_current_document()
    doc.document_start()

def document_end(w):
    doc = Moleskine.app.get_current_document()
    doc.document_end()

def page_up(w):
    doc = Moleskine.app.get_current_document()
    doc.page_up()

def page_down(w):
    doc = Moleskine.app.get_current_document()
    doc.page_down()

def word_right(w):
    doc = Moleskine.app.get_current_document()
    doc.word_right()

def word_left(w):
    doc = Moleskine.app.get_current_document()
    doc.word_left()

def backspace(w):
    doc = Moleskine.app.get_current_document()
    doc.delete_back()

def delete_forwards(w):
    doc = Moleskine.app.get_current_document()
    doc.clear()

def escape(w):
    pass

def enter(w):
    """insert the appropriate line-ending depending on the document's mode"""
    import GTKSCINTILLA
    doc = Moleskine.app.get_current_document()
    mode = doc.get_eol_mode()
    if mode == GTKSCINTILLA.EOL_LF:
        text = "\n"
    elif mode == GTKSCINTILLA.EOL_CRLF:
        text = "\r\n"
    else:#EOL_CR
        text = "\r"
    pos = doc.get_current_pos()
    doc.insert_text(pos, text)

def insert_or_overtype(w):
    """Flip mode to insert or overtype depending on previous state"""
    doc = Moleskine.app.get_current_document()
    doc.set_overtype(not doc.get_overtype())

def line_up_extend(w):
    doc = Moleskine.app.get_current_document()
    doc.line_up_extend()

def line_down_extend(w):
    doc = Moleskine.app.get_current_document()
    doc.line_down_extend()

def char_left_extend(w):
    doc = Moleskine.app.get_current_document()
    doc.char_left_extend()

def char_right_extend(w):
    doc = Moleskine.app.get_current_document()
    doc.char_right_extend()

def word_left_extend(w):
    doc = Moleskine.app.get_current_document()
    doc.word_left_extend()

def word_right_extend(w):
    doc = Moleskine.app.get_current_document()
    doc.word_right_extend()

def line_start_extend(w):
    doc = Moleskine.app.get_current_document()
    doc.home_extend()

def line_end_extend(w):
    doc = Moleskine.app.get_current_document()
    doc.line_end_extend()

def document_start_extend(w):
    doc = Moleskine.app.get_current_document()
    doc.document_start_extend()

def document_end_extend(w):
    doc = Moleskine.app.get_current_document()
    doc.document_end_extend()

def page_up_extend(w):
    doc = Moleskine.app.get_current_document()
    doc.page_up_extend()

def page_down_extend(w):
    doc = Moleskine.app.get_current_document()
    doc.page_down_extend()

#valid prefixes for multi-key commands
prefixes = [
    (CTRL, None, None, GDK.x),
    (CTRL, None, None, GDK.c)
    ]

#tuples are:
#((CTRL, ALT, SHIFT, keyval),(CTRL, ALT, SHIFT, keyval))
#with tuple beings last_values followed by current_values

emacs_keys = {
    ((CTRL, None, None, GDK.x),(CTRL, None, None, GDK.c)) : menu.exit,
    ((CTRL, None, None, GDK.x),(CTRL, None, None, GDK.f)) : menu.open_document,
    ((CTRL, None, None, GDK.x),(CTRL, None, None, GDK.s)) : menu.save_document,
    ((CTRL, None, None, GDK.x),(CTRL, None, None, GDK.w)) : menu.save_as_document,
    ((CTRL, None, None, GDK.x),(None, None, None, GDK.h)) : select_all,
    ((None, None, None, None),(CTRL, None, None, GDK.k)) : kill_line,
    ((None, None, None, None),(CTRL, None, None, GDK.Delete)) : kill_line,
    ((None, None, None, None),(CTRL, None, None, GDK.y)) : menu.paste,
    ((None, None, None, None),(CTRL, None, SHIFT, GDK.underscore)) : menu.undo,
    ((None, None, None, None),(None, ALT, None, GDK.g)) : menu.goto_line,
    ((None, None, None, None),(CTRL, None, None, GDK.p)): line_up,
    ((None, None, None, None),(None, None, None, GDK.Up)): line_up,
    ((None, None, None, None),(CTRL, None, None, GDK.n)): line_down,
    ((None, None, None, None),(None, None, None, GDK.Down)): line_down,
    ((None, None, None, None),(CTRL, None, None, GDK.b)): char_left,
    ((None, None, None, None),(None, None, None, GDK.Left)): char_left,
    ((None, None, None, None),(CTRL, None, None, GDK.f)): char_right,
    ((None, None, None, None),(None, None, None, GDK.Right)): char_right,
    ((None, None, None, None),(CTRL, None, None, GDK.e)): line_end,
    ((None, None, None, None),(None, None, None, GDK.End)): line_end,
    ((None, None, None, None),(CTRL, None, None, GDK.a)): line_start,
    ((None, None, None, None),(None, None, None, GDK.Home)): line_start,
    ((None, None, None, None),(CTRL, None, None, GDK.Home)):document_start,
    ((None, None, None, None),(CTRL, None, None, GDK.End)):document_end,
    ((None, None, None, None),(CTRL, None, None, GDK.v)): page_up,
    ((None, None, None, None),(None, None, None, GDK.Page_Up)): page_up,
    ((None, None, None, None),(None, ALT, None, GDK.v)): page_down,
    ((None, None, None, None),(None, None, None, GDK.Page_Down)): page_down,
    ((None, None, None, None),(None, ALT, None, GDK.f)): word_right,
    ((None, None, None, None),(CTRL, None, None, GDK.Right)): word_right,
    ((None, None, None, None),(None, ALT, None, GDK.b)): word_left,
    ((None, None, None, None),(CTRL, None, None, GDK.Left)): word_left,
    ((None, None, None, None),(None, None, None, GDK.BackSpace)):backspace,
    ((None, None, None, None),(CTRL, None, None, GDK.d)):delete_forwards,
    ((None, None, None, None),(None, None, None, GDK.Delete)):delete_forwards,
    ((None, None, None, None),(None, ALT, None, GDK.slash)):menu.complete_word,
    ((None, None, None, None),(None, None, None, GDK.Escape)):escape,
    ((None, None, None, None),(None, None, None, GDK.Return)):enter,
    ((None, None, None, None),(None, None, None, GDK.Insert)):insert_or_overtype,
    ((None, None, None, None),(None, None, SHIFT, GDK.Up)):line_up_extend,
    ((None, None, None, None),(None, None, SHIFT, GDK.Down)):line_down_extend,
    ((None, None, None, None),(None, None, SHIFT, GDK.Left)):char_left_extend,
    ((None, None, None, None),(None, None, SHIFT, GDK.Right)):char_right_extend,
    ((None, None, None, None),(CTRL, None, SHIFT, GDK.Left)):word_left_extend,
    ((None, None, None, None),(CTRL, None, SHIFT, GDK.Right)):word_right_extend,
    ((None, None, None, None),(None, None, SHIFT, GDK.Home)):line_start_extend,
    ((None, None, None, None),(None, None, SHIFT, GDK.End)):line_end_extend,
    ((None, None, None, None),(CTRL, None, SHIFT, GDK.Home)):document_start_extend,
    ((None, None, None, None),(CTRL, None, SHIFT, GDK.End)):document_end_extend,
    ((None, None, None, None),(None, None, SHIFT, GDK.Page_Up)):page_up_extend,
    ((None, None, None, None),(None, None, SHIFT, GDK.Page_Down)):page_down_extend,
}

def key_press(w, current_key):
    global last_values, CTRL, ALT, SHIFT
    l = []

    #is ctrl pressed
    if current_key.state & GDK.CONTROL_MASK:
        l.append(CTRL)
    else:
        l.append(None)
        
    #is alt pressed
    if current_key.state & GDK.MOD1_MASK:
        l.append(ALT)
    else:
        l.append(None)
        
    #is shift pressed
    if current_key.state & GDK.SHIFT_MASK:
        l.append(SHIFT)
    else:
        l.append(None)
        
    #identify key pressed
    l.append(current_key.keyval)
    
    current_values = tuple(l)
    cmd = (last_values, current_values)

    #process key if it's registered
    if emacs_keys.has_key(cmd):
        func = emacs_keys[cmd]
        func(w)
        w.emit_stop_by_name('key_press_event')
        last_values = (None, None, None, None)
        return

    #if key is a valid prefix, store it, otherwise ignore it
    else:
        if current_values in prefixes:
            last_values = current_values
            return
        else:
            last_values = (None, None, None, None)
            return


##############################################################################
# We must redefine the menus, sice most of the default key bindings clash with
# emacs key strokes
#
# Idealy we would just change the bindings of the existing menu items but
# I dont know how to get to them
from gnome.ui import *
from gnome.uiconsts import *

file_menu = [
    UIINFO_ITEM_STOCK(_('_New'), _('Create a new file'),
                       menu.new_document,STOCK_MENU_NEW),
    UIINFO_ITEM_STOCK(_('_Open...'), _('Load a file from disk'),
                       menu.open_document, STOCK_MENU_OPEN),
    UIINFO_SUBTREE_STOCK(menu.MENU_OPEN_PREVIOUS, [], STOCK_MENU_BOOK_OPEN),
    UIINFO_SEPARATOR,
    UIINFO_MENU_SAVE_ITEM(menu.save_document),
    UIINFO_MENU_SAVE_AS_ITEM(menu.save_as_document),
    UIINFO_ITEM_STOCK(_('Save all'), None, menu.save_all, STOCK_MENU_SAVE),
    UIINFO_MENU_REVERT_ITEM(menu.revert_document),
    UIINFO_SEPARATOR,
    (APP_UI_ITEM, _('Load session...'), _('Load session from a file'),
     menu.load_session, None, APP_PIXMAP_STOCK, STOCK_MENU_TIMER, GDK.F11, 0),
    (APP_UI_ITEM, _('Save session...'), _('Save current session to a file'),
     menu.save_session, None, APP_PIXMAP_STOCK, STOCK_MENU_TIMER_STOP, GDK.F12, 0),
    UIINFO_SUBTREE_STOCK(MENU_RECENT_SESSIONS, [], STOCK_MENU_BOOK_OPEN),
    UIINFO_SEPARATOR,
    UIINFO_ITEM_STOCK(_('Next'), None, menu.file_next, STOCK_MENU_FORWARD),
    UIINFO_ITEM_STOCK(_('Previous'), None, menu.file_previous, STOCK_MENU_BACK),
    (APP_UI_ITEM, _('Last used'), None, menu.file_last_used, None,
     APP_PIXMAP_NONE, None, GDK.P, GDK.CONTROL_MASK),
    UIINFO_SEPARATOR,
    UIINFO_ITEM_STOCK(_('_Close'), None, menu.close_document, STOCK_MENU_CLOSE),
    UIINFO_ITEM_STOCK(_('Close all'), None, menu.close_all, STOCK_MENU_CLOSE),
    UIINFO_ITEM_STOCK(_('E_xit'), None, menu.exit, STOCK_MENU_EXIT)
]

edit_menu = [
    (APP_UI_ITEM, _('_Undo'), _('Undo last action'),
     menu.undo, None, APP_PIXMAP_STOCK, STOCK_MENU_UNDO,
     ord('_'), GDK.CONTROL_MASK),
    UIINFO_ITEM_STOCK(_('_Redo'), _('Redo last action'), menu.redo, STOCK_MENU_REDO),
    UIINFO_SEPARATOR,
    (APP_UI_ITEM, _('C_ut'), _('Cut the current selection'),
     menu.cut, None, APP_PIXMAP_STOCK, STOCK_MENU_CUT,
     ord('w'), GDK.CONTROL_MASK),
    UIINFO_ITEM_STOCK(_('_Copy'), None, menu.copy, STOCK_MENU_COPY ),
    (APP_UI_ITEM, _('_Paste'), _('Paste from clipboard'),
     menu.paste, None, APP_PIXMAP_STOCK, STOCK_MENU_PASTE,
     ord('y'), GDK.CONTROL_MASK),
    UIINFO_SEPARATOR,
    (APP_UI_ITEM, _('_Search...'), _('Search a string in current file'),
     menu.search, None, APP_PIXMAP_STOCK, STOCK_MENU_SEARCH,
     ord('s'), GDK.CONTROL_MASK),
    (APP_UI_ITEM, _('Search in _files...'), _('Search a string in multiple files'),
     menu.search_in_files, None, APP_PIXMAP_STOCK, STOCK_MENU_SEARCH,
     0, 0),
    (APP_UI_ITEM, _('Search _Next'), _('Search the next occurrence of the last searched string'),
     menu.search_next, None, APP_PIXMAP_STOCK, STOCK_MENU_SEARCH,
     ord('g'), GDK.CONTROL_MASK),
    (APP_UI_ITEM, _('Search _Previous'), _('Search the previous occurrence of the last searched string'),
     menu.search_previous, None, APP_PIXMAP_STOCK, STOCK_MENU_SEARCH,
     ord('g'), GDK.CONTROL_MASK + GDK.SHIFT_MASK),
    (APP_UI_ITEM, _('_Replace...'), _('Search and replace a string in the current file'),
     menu.replace, None, APP_PIXMAP_STOCK, STOCK_MENU_SRCHRPL,
     ord('r'), GDK.CONTROL_MASK),
    UIINFO_SEPARATOR,
    (APP_UI_ITEM, _('_Goto line...'), _('Go to specified line in current file'),
     menu.goto_line, None, APP_PIXMAP_STOCK, STOCK_MENU_JUMP_TO,
     GDK.l, GDK.CONTROL_MASK + GDK.MOD1_MASK),
]

settings_menu = [
    UIINFO_ITEM_STOCK(_('_Configuration'), None, menu.options, STOCK_MENU_PROP),
    UIINFO_SEPARATOR,
    UIINFO_ITEM_STOCK(_('Languages _settings...'), None,
                      menu.lang_settings, STOCK_MENU_PREF),
    UIINFO_SUBTREE_STOCK(menu.MENU_CHANGE_LANGUAGE, [], STOCK_MENU_JUMP_TO),
    UIINFO_SEPARATOR,
    (APP_UI_TOGGLEITEM, _('Show _white space'), None, menu.toggle_ws, None,
     APP_PIXMAP_NONE, None, 0, 0),
    (APP_UI_TOGGLEITEM, _('Show _end of line'), None, menu.toggle_eol, None,
     APP_PIXMAP_NONE, None, 0, 0),
]

tools_menu = [
    (APP_UI_ITEM, _('_Next bookmark'), None, menu.next_bookmark, None,
     APP_PIXMAP_STOCK, STOCK_MENU_FORWARD, 0, 0),
    (APP_UI_ITEM, _('_Previous bookmark'), None, menu.prev_bookmark, None,
     APP_PIXMAP_STOCK, STOCK_MENU_BACK, 0, 0),
    (APP_UI_ITEM, _('_Toggle bookmark'), None, menu.toggle_bookmark, None,
     APP_PIXMAP_STOCK, STOCK_MENU_BOOK_YELLOW, 0, 0),
    UIINFO_ITEM_STOCK(_('_Clear bookmarks'), None,
                      menu.clear_bookmarks, STOCK_MENU_TRASH_FULL),
    UIINFO_SEPARATOR,
    (APP_UI_ITEM, _('_Complete word'),
     '',
     menu.complete_word, None, APP_PIXMAP_STOCK, STOCK_MENU_JUMP_TO,
     GDK.space, GDK.CONTROL_MASK),
    UIINFO_SUBTREE(_('Convert _EOLs'), [
        UIINFO_ITEM(_('LF (_Unix)'), None, menu.eol_to_lf),
        UIINFO_ITEM(_('CR (_Mac)'), None, menu.eol_to_cr),
        UIINFO_ITEM(_('CR+LF (Dos/_Win)'), None, menu.eol_to_crlf)]),
    UIINFO_SEPARATOR,
    UIINFO_SUBTREE(_('CVS'), [
        UIINFO_ITEM(_('Commit file to CVS'), None, menu.cvs_commit),
        UIINFO_ITEM(_('Update file from CVS'), None, menu.cvs_update),
        UIINFO_ITEM(_('CVS diff file'), None, menu.cvs_diff),
        UIINFO_ITEM(_('File\'s CVS status'), None, menu.cvs_status),
        UIINFO_ITEM(_('File\'s CVS log'), None, menu.cvs_log),
        UIINFO_ITEM(_('Add file to CVS'), None, menu.cvs_add),
        UIINFO_ITEM(_('Remove file from CVS'), None, menu.cvs_remove),
        UIINFO_SEPARATOR,
        UIINFO_ITEM(_('Commit directory to CVS'), None, menu.cvs_commit_dir),
        UIINFO_ITEM(_('Update directory from CVS'), None, menu.cvs_update_dir),
        UIINFO_ITEM(_('CVS diff directory'), None, menu.cvs_diff_dir),
        UIINFO_ITEM(_('Directory\'s CVS status'), None, menu.cvs_status_dir),
        UIINFO_ITEM(_('Directory\'s CVS log'), None, menu.cvs_log_dir),
        UIINFO_ITEM(_('Add directory to CVS'), None, menu.cvs_add_dir)#,
        #UIINFO_ITEM(_('Remove directory from CVS'), None, menu.cvs_directory)
        ]),
    UIINFO_SEPARATOR,
    (APP_UI_ITEM, _('Clean-up recent files list'), None, menu.clean_up_mru, None,
     APP_PIXMAP_NONE, None, 0, 0),
    (APP_UI_ITEM, _('Delete recent files list'), None, menu.delete_mru, None,
     APP_PIXMAP_NONE, None, 0, 0),
]

help_menu = [
#    UIINFO_MENU_ABOUT_ITEM(menu.about)
    UIINFO_ITEM_STOCK(_('Moleskine info'), None, menu.about, STOCK_MENU_ABOUT)
]

main_menu = [
    UIINFO_SUBTREE(menu.MENU_FILE, file_menu),
    UIINFO_SUBTREE(_('_Edit'), edit_menu),
    UIINFO_SUBTREE(menu.MENU_SETTINGS, settings_menu),
    UIINFO_SUBTREE(_('_Tools'), tools_menu),
    UIINFO_SUBTREE(_('_Help'), help_menu)
]



# find.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2000 - 2002   Michele Campeotto <micampe@micampe.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import Moleskine
import menu

text = None
new_text = None
flags = 0

def find_next():
    if text is None:
        from Dialogs import FindDialog
        FindDialog()
        return
    
    doc = Moleskine.app.get_current_document()
    
    if doc.get_selection_start() != doc.get_selection_end():
        doc.goto_pos(doc.get_selection_end())
    doc.search_anchor()
    doc.search_next(flags, text)
    doc.goto_pos(doc.get_selection_start())
    doc.search_next(flags, text)
    
    if doc.get_selection_start() != doc.get_selection_end():
        return 1
    else:
        return 0

def find_prev():
    if text is None:
        from Dialogs import FindDialog
        FindDialog()
        return
    
    doc = Moleskine.app.get_current_document()
    
    old_pos = doc.get_selection_start()
    if doc.get_selection_start() != doc.get_selection_end():
        doc.goto_pos(doc.get_selection_start() - 1)
    doc.search_anchor()
    doc.search_prev(flags, text)
    doc.goto_pos(doc.get_selection_start())
    doc.search_prev(flags, text)
    
    if doc.get_selection_start() != doc.get_selection_end():
        return 1
    else:
        doc.goto_pos(old_pos)
        return 0

def replace_next():
    if text is None or new_text is None:
        from Dialogs import ReplaceDialog
        ReplaceDialog()
        return
    
    doc = Moleskine.app.get_current_document()
    
    if doc.get_selection_start() == doc.get_selection_end():
        if not find_next():
            return
    doc.replace_sel(new_text)



# Document.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2000 - 2002   Michele Campeotto <micampe@micampe.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os.path
import string
import re
import stat

from gtk import *
from gnome.ui import *
from gnome.util import *

import Moleskine
import gtkscintilla
import GTKSCINTILLA

# Dummy class used to cache the find options between searches
class FindFlags:
    match_case = 0
    whole_words = 0
    word_start = 0
    regexp = 0
    backwards = 0
    text = ''
    
    replace_prompt = 1
    replace_all = 0
    replace_selection = 0
    replace_text = ''
    
    found = 0
    
    in_open_files = 1
    in_directory_files = 0
    recursive_directory = 1
    in_all_files = 1
    in_pattern_files = 0
    in_files_pattern = ''
    in_directory = ''

class Document(gtkscintilla.GtkScintilla):
    def __init__(self, name = _('Unnamed')):
        gtkscintilla.GtkScintilla.__init__(self)
        
        self.connect('save_point_left', self.__save_point_left)
        self.connect('save_point_reached', self.__save_point_reached)
        self.connect('update_ui', self.highligh_braces)
        self.connect('char_added', self.char_added)
        self.connect('uri_dropped', self.uri_dropped)
        
        self.set_margin_width_n(1, 0)
        self.set_margin_width_n(2, 0)
        self.autoc_set_choose_single(TRUE)
        self.set_tab_indents(1)
        self.set_backspace_unindents(1)
        self.set_caret_policy(0x01 | 0x04 | 0x08, 0)
        self.set_visible_policy(0x01 | 0x04, 3)
        
        self.setup_default_style()
        self.setup_caret_style()
        self.setup_line_numbers_style()
        self.setup_braces_style()
        self.setup_line_numbers_digits()
        self.set_language(Moleskine.app.langs['Plain text'])
        
        self.is_modified = self.get_modify
        self.do_auto_indent = FALSE
        self.filepath = None
        self.filename = None
        self.mtime = 0
        self.title = _('Unnamed')
        
        self.label = GtkLabel('%i. %s' % (len(Moleskine.app.documents) + 1, name))
        
        self.set_usize(320, 240)
        
        #if we're using emacs keybindings then remove all scintilla command keys
        if int(Moleskine.app.prefs['keyboard/keybindings']) == 2:
            self.clear_all_cmd_keys()

    def setup_default_style(self):
        self.style_reset_default()
        self.style_set_font(GTKSCINTILLA.STYLE_DEFAULT,
            Moleskine.app.prefs['default style/font'])
        self.style_set_fore(GTKSCINTILLA.STYLE_DEFAULT,
            int(Moleskine.app.prefs['default style/fore']))
        self.style_set_back(GTKSCINTILLA.STYLE_DEFAULT,
            int(Moleskine.app.prefs['default style/back']))
        self.set_sel_back(TRUE,
            int(Moleskine.app.prefs['default style/selection']))
        self.marker_set_back(TRUE,
            int(Moleskine.app.prefs['default style/bookmark']))
        self.style_clear_all()
    
    def setup_caret_style(self):
        self.set_caret_fore(int(Moleskine.app.prefs['caret/fore']))
        self.set_caret_width(int(Moleskine.app.prefs['caret/width']))
        self.set_caret_period(int(Moleskine.app.prefs['caret/period']))
    
    def setup_line_numbers_style(self):
        self.style_set_font(GTKSCINTILLA.STYLE_LINE_NUMBER,
                            Moleskine.app.prefs['line numbers/font'])
        self.style_set_fore(GTKSCINTILLA.STYLE_LINE_NUMBER,
                            int(Moleskine.app.prefs['line numbers/fore']))
        self.style_set_back(GTKSCINTILLA.STYLE_LINE_NUMBER,
                            int(Moleskine.app.prefs['line numbers/back']))
    
    def setup_braces_style(self):
        from Languages import _merge_font
        
        self.style_set_font(GTKSCINTILLA.STYLE_BRACE_LIGHT,
                            _merge_font('-*-*-bold-*-*-*-*-*-*-*-*-*-*-*'))
        self.style_set_fore(GTKSCINTILLA.STYLE_BRACE_LIGHT,
                            int(Moleskine.app.prefs['braces style/fore_ok']))
        self.style_set_back(GTKSCINTILLA.STYLE_BRACE_LIGHT,
                            int(Moleskine.app.prefs['default style/back']))
        
        self.style_set_font(GTKSCINTILLA.STYLE_BRACE_BAD,
                            _merge_font('-*-*-bold-*-*-*-*-*-*-*-*-*-*-*'))
        self.style_set_fore(GTKSCINTILLA.STYLE_BRACE_BAD,
                            int(Moleskine.app.prefs['braces style/fore_bad']))
        self.style_set_back(GTKSCINTILLA.STYLE_BRACE_BAD,
                            int(Moleskine.app.prefs['default style/back']))
    
    def setup_line_numbers_digits(self):
        def digits_width(digits):
            try:
                font = load_font(Moleskine.app.prefs['line numbers/font'])
            except RuntimeError:
                font = load_font('fixed')
            return font.width('0' * (digits + 1))
        
        if int(Moleskine.app.prefs['line numbers/show']):
            if int(Moleskine.app.prefs['line numbers/auto']):
                digits = len(str(self.get_line_count())) + 1
            else:
                digits = int(Moleskine.app.prefs['line numbers/digits'])
            self.set_margin_width_n(0, digits_width(digits))
        else:
            self.set_margin_width_n(0, 0)
    
    def update_title(self):
        def cmp(a, b):
            if len(a[1]) > len(b[1]):
                return -1
            elif len(a[1]) == len(b[1]):
                return 0
            else:
                return 1
        
        if self.filepath is None:
            return
        self.title = self.filepath
        Moleskine.app.prefs.dirnames.sort(cmp)
        for dirname in Moleskine.app.prefs.dirnames:
            if string.find(self.title, dirname[1]) == 0:
                self.title = string.replace(self.title, dirname[1], dirname[0])
                break
    
    def load_file(self, filepath):
        def read(filepath):
            file = open(filepath, 'r')
            text = file.read()
            file.close()
            return text
        
        if os.access(filepath, os.F_OK) and not os.access(filepath, os.R_OK):
            gnome.ui.GnomeMessageBox(
                _('Cannot open `%s\': permission denied.') % \
                os.path.basename(filepath),
                gnome.uiconsts.MESSAGE_BOX_ERROR,
                gnome.uiconsts.STOCK_BUTTON_OK).run_and_close()
            return
        if os.path.exists(filepath):
            self.mtime = os.stat(filepath)[stat.ST_MTIME]
            text = read(filepath)
            #self.clear_all()
            #self.add_text(len(text), text)
            self.set_text(text)
        self.filepath = filepath
        self.filename = os.path.basename(filepath)
        self.label.set_text('%i. %s' %
                                (Moleskine.app.notebook.get_current_page() + 1,
                                 self.filename))
        self.update_title()
        
        self.setup_line_numbers_digits()
        self.check_file_types()
        self.empty_undo_buffer()
        self.set_save_point()
    
    def save_file(self, filepath=None):
        def write(filepath, text):
            file = open(filepath, 'w')
            file.write(text)
            file.close()
        
        if filepath is not None:
            checkfilepath = filepath
        else:
            checkfilepath = self.filepath
        file_ok = 0
        if not os.access(checkfilepath, os.F_OK):
            if os.access(os.path.dirname(checkfilepath), os.W_OK):
                file_ok = 1
        else:
            if os.access(checkfilepath, os.W_OK):
                file_ok = 1
        if not file_ok:
            gnome.ui.GnomeMessageBox(
                _('File can not be written: %s\n\nPermission denied.') %
                    checkfilepath,
                gnome.uiconsts.MESSAGE_BOX_ERROR,
                gnome.uiconsts.STOCK_BUTTON_OK).run_and_close()
            return
        if filepath is not None:
            self.filepath = filepath
            self.filename = os.path.basename(filepath)
            self.label.set_text('%i. %s' %
                                (Moleskine.app.notebook.get_current_page() + 1,
                                 self.filename))
            self.update_title()
        text = self.get_text()
        write(self.filepath, text)
        self.mtime = os.stat(self.filepath)[stat.ST_MTIME]
        self.check_file_types()
        self.set_save_point()
    
    def revert(self):
        if self.filepath is not None:
            #doc = Moleskine.app.get_current_document()
            pos = self.get_current_pos()
            self.load_file(self.filepath)
            self.goto_pos(pos)
    
    def matches(self, lang):
        from fnmatch import fnmatch
        
        first_line = self.get_line(0)
        for type, rule in lang.rules:
            if type == 0:
                if fnmatch(self.filename, rule):
                    return TRUE
            elif rule == 1:
                if re.match(rule, filename):
                    return TRUE
            else:
                if re.match(rule, first_line):
                    return TRUE
        return FALSE
    
    def check_file_types(self):
        for lang in Moleskine.app.langs.values():
            if self.matches(lang):
                self.set_language(lang)
                break
    
    def set_language(self, language):
        def set_font(style, self=self):
            from Languages import _merge_font
            
            value = self.language.styles[style]['font']
            if value is not None:
                self.style_set_font(style, _merge_font(value))
            else:
                self.style_set_font(style,
                        Moleskine.app.prefs['default style/font'])
        
        def set_fore(style, self=self):
            value = self.language.styles[style]['fore']
            if value is not None:
                self.style_set_fore(style, int(value))
            else:
                self.style_set_fore(style,
                        int(Moleskine.app.prefs['default style/fore']))
        
        def set_back(style, self=self):
            value = self.language.styles[style]['back']
            if value is not None:
                self.style_set_back(style, int(value))
            else:
                self.style_set_back(style,
                        int(Moleskine.app.prefs['default style/back']))
        
        self.language = language
        
        self.set_tab_width(language.tab_size)
        self.set_use_tabs(not language.soft_tabs)
        self.set_indent(language.indent_size)
        self.set_indentation_guides(language.show_indent)
        self.set_auto_indent(language.auto_indent)
        self.set_edge_mode(language.edge_indicator)
        self.set_edge_column(language.edge_column)
        self.set_edge_colour(language.edge_color)
        
        if language.lexer == GTKSCINTILLA.LEXER_HTML or \
           language.lexer == GTKSCINTILLA.LEXER_XML:
            self.set_style_bits(7)
        else:
            self.set_style_bits(5)
        self.set_lexer(language.lexer)
        self.clear_document_style()
        
        for kwclass in range(5):
            if language.keywords[kwclass] is not None:
                self.set_keywords(kwclass, language.keywords[kwclass])
        
        for i in language.styles.keys():
            set_font(i)
            set_fore(i)
            set_back(i)
        
        self.colourise(0, -1)
        self.load_api()
    
    def load_api(self):
        if self.language.api_file != '' and self.language.api_file is not None:
            self.api = {}
            api_file = open(self.language.api_file)
            for line in api_file.readlines():
                self.api[line[:string.find(line, '(')]] = line[:-1]
    
    def reload_language(self):
        self.set_language(self.language)
    
    def set_auto_indent(self, do_indent):
        self.do_auto_indent = do_indent
    
    def auto_indent(self):
        line = self.get_current_line() - 1
        if line > 0:
            indent = self.get_line_indentation(line)
            line = self.get_current_line()
            self.begin_undo_action()
            self.set_line_indentation(line, indent)
            self.goto_pos(self.get_line_indent_position(line))
            self.end_undo_action()
    
    def get_current_word(self):
        """Return the word the cursor is on"""
        pos = self.get_current_pos()
        self.word_left_extend()
        word = self.get_sel_text()
        self.goto_pos(pos)

        return word
    
    def get_function_name(self):
        pos = self.get_current_pos()
        self.goto_pos(pos - 1)
        self.word_left_extend()
        word = string.strip(self.get_sel_text())
        self.goto_pos(pos)
        return word
    
    def find_words(self, curword):
        """Find all words starting with a given prefix
        
        Find all words in the text starting with a given prefix.
        
        curword -- the prefix
        
        return -- a string with all the words separated by a space
        """
        text = self.get_text()
        expr = re.compile('(?:\A|\W)(' + curword + '\w*)')
        words = expr.findall(text)
        if len(words) > 0:
            d = {}                  # Remove duplicated items from words list
            for w in words:
                d[w] = None
            words = d.keys()
            try:
                words.extend(filter(expr.match, self.api.keys()))
            except AttributeError:
                pass
            words.sort()
            if curword in words:    # Remove the current word from the list
                words.remove(curword)
            return string.join(words)
        else:
            return None
    
    def complete_word(self):
        self.replace_sel('')
        curword = self.get_current_word()
        words = self.find_words(curword)
        if words:
            self.autoc_show(len(curword), words)
    
    def char_added(self, w, char, user_data=None):
        if self.do_auto_indent and (char == 10 or char == 13):
            self.auto_indent()
        
        if Moleskine.app.prefs['complete/auto'] == TRUE and \
           chr(char) in string.letters + '_' and not self.autoc_active():
            curword = self.get_current_word()
            words = self.find_words(curword)
            if words:
                curpos = self.get_current_pos()
                word = string.split(words)[0]
                the_rest = word[len(curword):]
                self.insert_text(curpos, the_rest)
                self.set_sel(curpos, curpos + len(the_rest))
        
        if chr(char) == '(':
            curpos = self.get_current_pos()
            try:
                function = self.get_function_name()
                description = self.api[function]
                description = string.replace(description, ') ', ')\n')
                self.call_tip_show(curpos - len(function), description)
            except:
                pass
        elif chr(char) == ')' and self.call_tip_active:
            self.call_tip_cancel()
    
    def uri_dropped(self, w, uri, user_data=None):
        uris = string.split(uri[:-1], '\n')
        for file in uris:
            Moleskine.app.open_document(file)
    
    def is_brace(self):
        position = self.get_current_pos()
        ch = self.get_char_at(position - 1)
        if ch in range(256) and chr(ch) in '[](){}':
            return position - 1
        ch = self.get_char_at(position)
        if ch in range(256) and chr(ch) in '[](){}':
            return position
        return -1

    def highligh_braces(self, w, user_data=None):
        brace_pos = self.is_brace()
        if brace_pos > 0:
            other_pos = self.brace_match(brace_pos)
            if other_pos > 0:
                self.brace_highlight(brace_pos, other_pos)
            else:
                self.brace_bad_light(brace_pos)
        else:
            self.brace_bad_light(-1)

    def can_close(self):
        return not self.is_modified()
    
    def is_empty(self):
        return self.get_text_length() == 0
    
    def get_current_line(self):
        return self.line_from_position(self.get_current_pos())
    
    def __save_point_left(self, w, user_data=None):
        if self.filename is not None:
            self.label.set_text('*%i. %s' %
                                (Moleskine.app.notebook.get_current_page() + 1,
                                 self.filename))
        else:
            self.label.set_text('*%i. %s' %
                                (Moleskine.app.notebook.get_current_page() + 1,
                                 _('Unnamed')))
        Moleskine.app.update_title()
    
    def __save_point_reached(self, w, user_data=None):
        if self.filename is not None:
            self.label.set_text('%i. %s' %
                                (Moleskine.app.notebook.get_current_page() + 1,
                                 self.filename))
        else:
            self.label.set_text('%i. %s' %
                                (Moleskine.app.notebook.get_current_page() + 1,
                                 _('Unnamed')))
        Moleskine.app.update_title()
    
    def search_text(self, text, match_case=0, whole_words=0,
                    word_start=0, regexp=0, backwards=0):
        """Search for a string in the active document
        
        The function starts searching from the current position. If the word
        is found the function selects the string and returns a tuple
        containing its starting and ending positions, or None if the string
        hasn't been found.
        """
        FindFlags.text = text
        FindFlags.match_case = match_case
        FindFlags.whole_words = whole_words
        FindFlags.word_start = word_start
        FindFlags.regexp = regexp
        FindFlags.backwards = backwards
        
        flags = backwards * GTKSCINTILLA.FIND_DOWN + \
                match_case * GTKSCINTILLA.FIND_MATCH_CASE + \
                whole_words * GTKSCINTILLA.FIND_WHOLE_WORDS + \
                word_start * GTKSCINTILLA.FIND_WORD_START + \
                regexp * GTKSCINTILLA.FIND_REGEXP
        if backwards:
            start = self.get_selection_start() - 1
            end = 0
        else:
            start = self.get_selection_end()
            end = self.get_length()
        result = self.find_text(flags, text, (start, end))
        if result:
            self.goto_pos(result[0])
            self.set_selection_start(result[0])
            self.set_selection_end(result[1])
            FindFlags.found = 1
        else:
            if (backwards):
                start = self.get_length ()
                end = self.get_selection_start() + 1
            else:
                start = 0
                end = self.get_selection_start() - 1
            result = self.find_text(flags, text, (start, end))
            if (result):
                self.goto_pos(result[0])
                self.set_selection_start(result[0])
                self.set_selection_end(result[1])
                FindFlags.found = 1
            else:
                self.set_selection_start(self.get_selection_end())
                FindFlags.found = 0
        return result
    
    def search(self):
        """Repeat the last performed search
        
        The function uses the same options and the same criterion used in the
        last search.
        """
        return self.search_text(text=FindFlags.text,
                                match_case=FindFlags.match_case,
                                whole_words=FindFlags.whole_words,
                                word_start=FindFlags.word_start,
                                regexp=FindFlags.regexp,
                                backwards=FindFlags.backwards)
    
    def search_next(self):
        """Search for the next string matching the last used criterion
        
        The function uses the same options and the same criterion used in the
        last search.
        """
        return self.search_text(text=FindFlags.text,
                                match_case=FindFlags.match_case,
                                whole_words=FindFlags.whole_words,
                                word_start=FindFlags.word_start,
                                regexp=FindFlags.regexp,
                                backwards=0)
    
    def search_previous(self):
        """Search for the previous string matching the last used criterion
        
        The function uses the same options and the same criterion used in the
        last search.
        """
        return self.search_text(text=FindFlags.text,
                                match_case=FindFlags.match_case,
                                whole_words=FindFlags.whole_words,
                                word_start=FindFlags.word_start,
                                regexp=FindFlags.regexp,
                                backwards=1)
    
    def replace_target(self, regexp, text):
        if regexp:
            return self.replace_target_re(len(text), text)
        else:
            return gtkscintilla.GtkScintilla.replace_target(self, len(text),
                                                            text)



# GladeDialog.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2000 - 2002   Michele Campeotto <micampe@micampe.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os.path

import gnome.ui
import gnome.util
import libglade

class GladeDialog(gnome.ui.GnomeDialog):
    
    """A GnomeDialog window contructed from a Glade file."""
    
    def __init__(self, name):
        """Create a new GladeDialog.
        
        'name' is the name (no path, no extension) of the glade file
        containing the dialog definition and the name of the root widget.
        """
        self.filepath = gnome.util.datadir_file( \
                os.path.join('Moleskine/dialogs', name + '.glade'))
        self.xml = libglade.GladeXML(self.filepath, name)
        dialog = self.xml.get_widget(name)
        gnome.ui.GnomeDialog.__init__(self, _obj = dialog._o)
        dict = {}
        for key in dir(self.__class__):
            dict[key] = getattr(self, key)
        self.xml.signal_autoconnect(dict)
    
    def __getattr__(self, key):
        w = self.xml.get_widget(key)
        if w is None:
            raise AttributeError(_("Widget %s not found") % key)
        self.__dict__[key] = w
        return w
    __getitem__ = __getattr__



# Languages.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2000 - 2002   Michele Campeotto <micampe@micampe.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from UserDict import *
import os.path
import string

from GTK import *
import gnome.config

import GTKSCINTILLA
import Moleskine

_CONFIG_FILE = '/Moleskine/LanguageModes/'

Lexers = {
    _('None')          : GTKSCINTILLA.LEXER_NULL,
    _('Python')        : GTKSCINTILLA.LEXER_PYTHON,
    _('C/C++')         : GTKSCINTILLA.LEXER_CPP,
    _('HTML')          : GTKSCINTILLA.LEXER_HTML,
    _('XML')           : GTKSCINTILLA.LEXER_XML,
    _('Perl')          : GTKSCINTILLA.LEXER_PERL,
    _('SQL')           : GTKSCINTILLA.LEXER_SQL,
    _('Visual Basic')  : GTKSCINTILLA.LEXER_VB,
    _('Properties')    : GTKSCINTILLA.LEXER_PROPERTIES,
    _('Error list')    : GTKSCINTILLA.LEXER_ERRORLIST,
    _('Makefile')      : GTKSCINTILLA.LEXER_MAKEFILE,
    _('Batch file')    : GTKSCINTILLA.LEXER_BATCH,
    _('LaTeX')         : GTKSCINTILLA.LEXER_LATEX,
    _('LUA')           : GTKSCINTILLA.LEXER_LUA,
    _('Diff')          : GTKSCINTILLA.LEXER_DIFF,
    _('Apache Config') : GTKSCINTILLA.LEXER_CONF,
    _('Pascal')        : GTKSCINTILLA.LEXER_PASCAL,
    _('Avenue')        : GTKSCINTILLA.LEXER_AVE,
    _('Ada')           : GTKSCINTILLA.LEXER_ADA,
    _('Lisp')          : GTKSCINTILLA.LEXER_LISP,
    _('Ruby')          : GTKSCINTILLA.LEXER_RUBY,
    _('Eiffel')        : GTKSCINTILLA.LEXER_EIFFEL,
    _('Eiffel KW')     : GTKSCINTILLA.LEXER_EIFFELKW,
    _('Tcl')           : GTKSCINTILLA.LEXER_TCL,
    _('NN Crontab')    : GTKSCINTILLA.LEXER_NNCRONTAB,
    _('Bullant')       : GTKSCINTILLA.LEXER_BULLANT,
    _('VB Script')     : GTKSCINTILLA.LEXER_VBSCRIPT,
    _('ASP')           : GTKSCINTILLA.LEXER_ASP,
    _('PHP')           : GTKSCINTILLA.LEXER_PHP,
    _('Baan')          : GTKSCINTILLA.LEXER_BAAN,
}

Styles = {
    GTKSCINTILLA.LEXER_PYTHON : {
        _('White space')               : GTKSCINTILLA.PYTHON_WHITE_SPACE,
        _('Comment line')              : GTKSCINTILLA.PYTHON_COMMENT_LINE,
        _('Number')                    : GTKSCINTILLA.PYTHON_NUMBER,
        _('String double quote')       : GTKSCINTILLA.PYTHON_STRING_DOUBLE,
        _('String single quote')       : GTKSCINTILLA.PYTHON_STRING_SINGLE,
        _('Keyword')                   : GTKSCINTILLA.PYTHON_KEYWORD,
        _('Long string double quote')  : GTKSCINTILLA.PYTHON_TRIPLE_DOUBLE,
        _('Long string single quote')  : GTKSCINTILLA.PYTHON_TRIPLE_SINGLE,
        _('Class name definition')     : GTKSCINTILLA.PYTHON_CLASS_STATEMENT,
        _('Function name definition')  : GTKSCINTILLA.PYTHON_DEF_STATEMENT,
        _('Operator')                  : GTKSCINTILLA.PYTHON_OPERATOR,
        _('Identifier')                : GTKSCINTILLA.PYTHON_IDENTIFIER,
        _('Comment block')             : GTKSCINTILLA.PYTHON_COMMENT_BLOCK,
        _('String EOL')                : GTKSCINTILLA.PYTHON_STRING_EOL },
    GTKSCINTILLA.LEXER_CPP : {
        _('White space')           : GTKSCINTILLA.CPP_WHITE_SPACE,
        _('Comment block')         : GTKSCINTILLA.CPP_COMMENT_BLOCK,
        _('Comment line')          : GTKSCINTILLA.CPP_COMMENT_LINE,
        _('Comment doc')           : GTKSCINTILLA.CPP_COMMENT_DOC,
        _('Number')                : GTKSCINTILLA.CPP_NUMBER,
        _('Keyword')               : GTKSCINTILLA.CPP_KEYWORD,
        _('String')                : GTKSCINTILLA.CPP_STRING,
        _('Character')             : GTKSCINTILLA.CPP_CHARACTER,
        _('UUID')                  : GTKSCINTILLA.CPP_UUID,
        _('Preprocessor')          : GTKSCINTILLA.CPP_PREPROCESSOR,
        _('Operator')              : GTKSCINTILLA.CPP_OPERATOR,
        _('Identifier')            : GTKSCINTILLA.CPP_IDENTIFIER,
        _('String not closed')     : GTKSCINTILLA.CPP_STRING_EOL,
        _('Verbatim String (C#)')  : GTKSCINTILLA.CPP_STRING_VERBATIM,
        _('Regular expression')    : GTKSCINTILLA.CPP_REGEX,
        _('Comment line doc')      : GTKSCINTILLA.CPP_COMMENT_LINE_DOC,
        _('Keyword 2')             : GTKSCINTILLA.CPP_KEYWORD2,
        _('Comment doc keyword')   : GTKSCINTILLA.CPP_COMMENT_DOC_KEYWORD,
        _('Comment doc error')     : GTKSCINTILLA.CPP_COMMENT_DOC_KEYWORD_ERROR },
    GTKSCINTILLA.LEXER_HTML : {
        _('Text')                  : GTKSCINTILLA.HTML_TEXT,
        _('Tag')                   : GTKSCINTILLA.HTML_TAG,
        _('Tag unknown')           : GTKSCINTILLA.HTML_TAG_UNKNOWN,
        _('Attribute')             : GTKSCINTILLA.HTML_ATTRIBUTE,
        _('Attribute unknown')     : GTKSCINTILLA.HTML_ATTRIBUTE_UNKNOWN,
        _('Number')                : GTKSCINTILLA.HTML_NUMBER,
        _('String double quote')   : GTKSCINTILLA.HTML_DOUBLE_STRING,
        _('String single quote')   : GTKSCINTILLA.HTML_SINGLE_STRING,
        _('Other')                 : GTKSCINTILLA.HTML_OTHER,
        _('Comment')               : GTKSCINTILLA.HTML_COMMENT,
        _('Entity')                : GTKSCINTILLA.HTML_ENTITY,
        _('XML tag ends (/>)')     : GTKSCINTILLA.HTML_TAG_END,
        _('Identifier start (<?)') : GTKSCINTILLA.HTML_XML_START,
        _('Identifier end (?>)')   : GTKSCINTILLA.HTML_XML_END,
        _('Script')                : GTKSCINTILLA.HTML_SCRIPT,
        _('ASP')                   : GTKSCINTILLA.HTML_ASP,
        _('ASP Alt')               : GTKSCINTILLA.HTML_ASP_AT,
        _('CDATA')                 : GTKSCINTILLA.HTML_CDATA,
        _('Question')              : GTKSCINTILLA.HTML_QUESTION,
        _('Value')                 : GTKSCINTILLA.HTML_VALUE,
        
        _('XCode comment')         : GTKSCINTILLA.HTML_XC_COMMENT,
        
        _('SGML default')           : GTKSCINTILLA.HTML_SGML_DEFAULT,
        _('SGML command')           : GTKSCINTILLA.HTML_SGML_COMMAND,
        _('SGML 1st param')         : GTKSCINTILLA.HTML_SGML_1ST_PARAM,
        _('SGML string double')     : GTKSCINTILLA.HTML_SGML_DOUBLE_STRING,
        _('SGML string single')     : GTKSCINTILLA.HTML_SGML_SIMPLE_STRING,
        _('SGML error')             : GTKSCINTILLA.HTML_SGML_ERROR,
        _('SGML special')           : GTKSCINTILLA.HTML_SGML_SPECIAL,
        _('SGML entity')            : GTKSCINTILLA.HTML_SGML_ENTITY,
        _('SGML comment')           : GTKSCINTILLA.HTML_SGML_COMMENT,
        _('SGML 1st param comment') : GTKSCINTILLA.HTML_SGML_1ST_PARAM_COMMENT,
        _('SGML block default')     : GTKSCINTILLA.HTML_SGML_BLOCK_DEFAULT,
        
        _('JS Start')                  : GTKSCINTILLA.HTML_JS_START,
        _('JS Default')                : GTKSCINTILLA.HTML_JS_DEFAULT,
        _('JS Comment block')          : GTKSCINTILLA.HTML_JS_COMMENT_BLOCK,
        _('JS Comment line')           : GTKSCINTILLA.HTML_JS_COMMENT_LINE,
        _('JS Comment doc')            : GTKSCINTILLA.HTML_JS_COMMENT_DOC,
        _('JS Number')                 : GTKSCINTILLA.HTML_JS_NUMBER,
        _('JS Word')                   : GTKSCINTILLA.HTML_JS_WORD,
        _('JS Keyword')                : GTKSCINTILLA.HTML_JS_KEYWORD,
        _('JS String double quote')    : GTKSCINTILLA.HTML_JS_STRING_DOUBLE,
        _('JS String single quote')    : GTKSCINTILLA.HTML_JS_STRING_SINGLE,
        _('JS Symbol')                 : GTKSCINTILLA.HTML_JS_SYMBOL,
        _('JS String EOL')             : GTKSCINTILLA.HTML_JS_EOL,
        
        _('PY Start')                      : GTKSCINTILLA.HTML_PY_START,
        _('PY Default')                    : GTKSCINTILLA.HTML_PY_DEFAULT,
        _('PY Comment line')               : GTKSCINTILLA.HTML_PY_COMMENT_LINE,
        _('PY Number')                     : GTKSCINTILLA.HTML_PY_NUMBER,
        _('PY String double quote')        : GTKSCINTILLA.HTML_PY_STRING_DOUBLE,
        _('PY String single quote')        : GTKSCINTILLA.HTML_PY_STRING_SINGLE,
        _('PY Keyword')                    : GTKSCINTILLA.HTML_PY_KEYWORD,
        _('PY Long string double quote')   : GTKSCINTILLA.HTML_PY_TRIPLE_SINGLE,
        _('PY Long string single quote')   : GTKSCINTILLA.HTML_PY_TRIPLE_DOUBLE,
        _('PY Class name definition')      : GTKSCINTILLA.HTML_PY_CLASS_STATEMENT,
        _('PY Function name definition')   : GTKSCINTILLA.HTML_PY_DEF_STATEMENT,
        _('PY Operator')                   : GTKSCINTILLA.HTML_PY_OPERATOR,
        _('PY Identifier')                 : GTKSCINTILLA.HTML_PY_IDENTIFIER,
        
        _('PHP Default')               : GTKSCINTILLA.HTML_PHP_DEFAULT,
        _('PHP String double quote')   : GTKSCINTILLA.HTML_PHP_STRING_DOUBLE,
        _('PHP String single quote')   : GTKSCINTILLA.HTML_PHP_STRING_SINGLE,
        _('PHP Keyword')               : GTKSCINTILLA.HTML_PHP_KEYWORD,
        _('PHP Number')                : GTKSCINTILLA.HTML_PHP_NUMBER,
        _('PHP Variable')              : GTKSCINTILLA.HTML_PHP_VARIABLE,
        _('PHP Comment block')         : GTKSCINTILLA.HTML_PHP_COMMENT_BLOCK,
        _('PHP Comment line')          : GTKSCINTILLA.HTML_PHP_COMMENT_LINE,
        _('PHP Variable in string')    : GTKSCINTILLA.HTML_PHP_STRING_VARIABLE,
        _('PHP Operator')              : GTKSCINTILLA.HTML_PHP_OPERATOR },
    GTKSCINTILLA.LEXER_XML : {
        _('Text')                  : GTKSCINTILLA.XML_DEFAULT,
        _('Tag')                   : GTKSCINTILLA.XML_TAG,
        _('Tag unknown')           : GTKSCINTILLA.XML_TAG_UNKNOWN,
        _('Attribute')             : GTKSCINTILLA.XML_ATTRIBUTE,
        _('Attribute unknown')     : GTKSCINTILLA.XML_ATTRIBUTE_UNKNOWN,
        _('Number')                : GTKSCINTILLA.XML_NUMBER,
        _('String double quote')   : GTKSCINTILLA.XML_DOUBLE_STRING,
        _('String single quote')   : GTKSCINTILLA.XML_SINGLE_STRING,
        _('Other')                 : GTKSCINTILLA.XML_OTHER,
        _('Comment')               : GTKSCINTILLA.XML_COMMENT,
        _('Entity')                : GTKSCINTILLA.XML_ENTITY,
        _('Tag ends (/>)')         : GTKSCINTILLA.XML_TAG_END,
        _('Identifier start (<?)') : GTKSCINTILLA.XML_START,
        _('Identifier end (?>)')   : GTKSCINTILLA.XML_SCRIPT,
        _('CDATA')                 : GTKSCINTILLA.XML_CDATA,
        _('Question')              : GTKSCINTILLA.XML_QUESTION },
    GTKSCINTILLA.LEXER_PERL : {
        _('White space')           : GTKSCINTILLA.PERL_WHITE_SPACE,
        _('Error')                 : GTKSCINTILLA.PERL_ERROR,
        _('Comment line')          : GTKSCINTILLA.PERL_COMMENT_LINE,
        _('POD')                   : GTKSCINTILLA.PERL_POD,
        _('Number')                : GTKSCINTILLA.PERL_NUMBER,
        _('Keyword')               : GTKSCINTILLA.PERL_KEYWORD,
        _('String')                : GTKSCINTILLA.PERL_STRING,
        _('Character')             : GTKSCINTILLA.PERL_CHARACTER,
        _('Punctuation')           : GTKSCINTILLA.PERL_PUNCTUATION,
        _('Preprocessor')          : GTKSCINTILLA.PERL_PREPROCESSOR,
        _('Operator')              : GTKSCINTILLA.PERL_OPERATOR,
        _('Identifier')            : GTKSCINTILLA.PERL_IDENTIFIER,
        _('Scalar')                : GTKSCINTILLA.PERL_SCALAR,
        _('Array')                 : GTKSCINTILLA.PERL_ARRAY,
        _('Hash')                  : GTKSCINTILLA.PERL_HASH,
        _('Symbol table')          : GTKSCINTILLA.PERL_SYMBOL_TABLE,
        _('Regular expression')    : GTKSCINTILLA.PERL_REGEX,
        _('Regex substitution')    : GTKSCINTILLA.PERL_REGSUBST,
        _('Long quote')            : GTKSCINTILLA.PERL_LONG_QUOTE,
        _('Backticks')             : GTKSCINTILLA.PERL_BACKTICKS,
        _('Data section')          : GTKSCINTILLA.PERL_DATA_SECTION,
        _('Here doc delimiter')    : GTKSCINTILLA.PERL_HERE_DELIM,
        _('Here doc Q')            : GTKSCINTILLA.PERL_HERE_Q,
        _('Here doc QQ')           : GTKSCINTILLA.PERL_HERE_QQ,
        _('Here doc QX')           : GTKSCINTILLA.PERL_HERE_QX,
        _('String Q')              : GTKSCINTILLA.PERL_STRING_Q,
        _('String QQ')             : GTKSCINTILLA.PERL_STRING_QQ,
        _('String QX')             : GTKSCINTILLA.PERL_STRING_QX,
        _('String QR')             : GTKSCINTILLA.PERL_STRING_QR,
        _('String QW')             : GTKSCINTILLA.PERL_STRING_QW },
    GTKSCINTILLA.LEXER_LATEX : {
        _('White space')   : GTKSCINTILLA.LATEX_WHITE_SPACE,
        _('Command')       : GTKSCINTILLA.LATEX_COMMAND,
        _('Tag')           : GTKSCINTILLA.LATEX_TAG,
        _('Math')          : GTKSCINTILLA.LATEX_MATH,
        _('Comment')       : GTKSCINTILLA.LATEX_COMMENT },
    GTKSCINTILLA.LEXER_LUA : {
        _('White space')       : GTKSCINTILLA.LUA_WHITE_SPACE,
        _('Comment block')     : GTKSCINTILLA.LUA_COMMENT_BLOCK,
        _('Comment line')      : GTKSCINTILLA.LUA_COMMENT_LINE,
        _('Comment doc')       : GTKSCINTILLA.LUA_COMMENT_DOC,
        _('Number')            : GTKSCINTILLA.LUA_NUMBER,
        _('Keyword')           : GTKSCINTILLA.LUA_KEYWORD,
        _('String')            : GTKSCINTILLA.LUA_STRING,
        _('Character')         : GTKSCINTILLA.LUA_CHARACTER,
        _('String literal')    : GTKSCINTILLA.LUA_LITERAL_STRING,
        _('Preprocessor')      : GTKSCINTILLA.LUA_PREPROCESSOR,
        _('Operator')          : GTKSCINTILLA.LUA_OPERATOR,
        _('Identifier')        : GTKSCINTILLA.LUA_IDENTIFIER,
        _('String EOL')        : GTKSCINTILLA.LUA_STRING_EOL },
    GTKSCINTILLA.LEXER_ERRORLIST : {
        _('White space')   : GTKSCINTILLA.ERROR_DEFAULT,
        _('Python')        : GTKSCINTILLA.ERROR_PYTHON,
        _('GCC')           : GTKSCINTILLA.ERROR_GCC,
        _('Microsoft')     : GTKSCINTILLA.ERROR_MS,
        _('Command')       : GTKSCINTILLA.ERROR_CMD,
        _('Borland')       : GTKSCINTILLA.ERROR_BORLAND,
        _('Perl')          : GTKSCINTILLA.ERROR_PERL,
        _('Diff changed')  : GTKSCINTILLA.ERROR_DIFF_CHANGED,
        _('Diff added')    : GTKSCINTILLA.ERROR_DIFF_ADDITION,
        _('Diff removed')  : GTKSCINTILLA.ERROR_DIFF_DELETION,
        _('Diff message')  : GTKSCINTILLA.ERROR_DIFF_MESSAGE },
    GTKSCINTILLA.LEXER_BATCH : {
        _('White space')   : GTKSCINTILLA.BATCH_DEFAULT,
        _('Comment')       : GTKSCINTILLA.BATCH_COMMENT,
        _('Keyword')       : GTKSCINTILLA.BATCH_WORD,
        _('Label')         : GTKSCINTILLA.BATCH_LABEL,
        _('Hide (@)')      : GTKSCINTILLA.BATCH_HIDE,
        _('Command')       : GTKSCINTILLA.BATCH_COMMAND,
        _('Variable')      : GTKSCINTILLA.BATCH_IDENTIFIER,
        _('Operator')      : GTKSCINTILLA.BATCH_OPERATOR },
    GTKSCINTILLA.LEXER_MAKEFILE : {
        _('Default')       : GTKSCINTILLA.MAKEFILE_WHITE_SPACE,
        _('Comment')       : GTKSCINTILLA.MAKEFILE_COMMENT,
        _('Preprocessor')  : GTKSCINTILLA.MAKEFILE_PREPROCESSOR,
        _('Variable')      : GTKSCINTILLA.MAKEFILE_IDENTIFIER,
        _('Operator')      : GTKSCINTILLA.MAKEFILE_OPERATOR,
        _('Error')         : GTKSCINTILLA.MAKEFILE_IDEOL },
    GTKSCINTILLA.LEXER_CONF : {
        _('White space')       : GTKSCINTILLA.CONF_DEFAULT,
        _('Comment')           : GTKSCINTILLA.CONF_COMMENT,
        _('Number')            : GTKSCINTILLA.CONF_NUMBER,
        _('Identifier')        : GTKSCINTILLA.CONF_IDENTIFIER,
        _('File extension')    : GTKSCINTILLA.CONF_EXTENSION,
        _('Parameter')         : GTKSCINTILLA.CONF_PARAMETER,
        _('String')            : GTKSCINTILLA.CONF_STRING,
        _('Operator')          : GTKSCINTILLA.CONF_OPERATOR,
        _('IP address')        : GTKSCINTILLA.CONF_IP,
        _('Directive')         : GTKSCINTILLA.CONF_DIRECTIVE },
    GTKSCINTILLA.LEXER_AVE : {
        _('White space')   : GTKSCINTILLA.AVE_DEFAULT,
        _('Comment')       : GTKSCINTILLA.AVE_COMMENT,
        _('Number')        : GTKSCINTILLA.AVE_NUMBER,
        _('Word')          : GTKSCINTILLA.AVE_WORD,
        _('Keyword')       : GTKSCINTILLA.AVE_KEYWORD,
        _('Statement')     : GTKSCINTILLA.AVE_STATEMENT,
        _('String')        : GTKSCINTILLA.AVE_STRING,
        _('Enum')          : GTKSCINTILLA.AVE_ENUM,
        _('String EOL')    : GTKSCINTILLA.AVE_STRINGEOL,
        _('Identifier')    : GTKSCINTILLA.AVE_IDENTIFIER,
        _('Operator')      : GTKSCINTILLA.AVE_OPERATOR },
    GTKSCINTILLA.LEXER_ADA : {
        _('White space')   : GTKSCINTILLA.ADA_DEFAULT,
        _('Comment')       : GTKSCINTILLA.ADA_COMMENT,
        _('Number')        : GTKSCINTILLA.ADA_NUMBER,
        _('Keyword')       : GTKSCINTILLA.ADA_WORD,
        _('String')        : GTKSCINTILLA.ADA_STRING,
        _('Character')     : GTKSCINTILLA.ADA_CHARACTER,
        _('Operator')      : GTKSCINTILLA.ADA_OPERATOR,
        _('Identifier')    : GTKSCINTILLA.ADA_IDENTIFIER,
        _('String EOL')    : GTKSCINTILLA.ADA_STRINGEOL },
    GTKSCINTILLA.LEXER_BAAN : {
    },
    GTKSCINTILLA.LEXER_LISP : {
        _('Default')       : GTKSCINTILLA.LISP_DEFAULT,
        _('Comment')       : GTKSCINTILLA.LISP_COMMENT,
        _('Number')        : GTKSCINTILLA.LISP_NUMBER,
        _('Keyword')       : GTKSCINTILLA.LISP_KEYWORD,
        _('String')        : GTKSCINTILLA.LISP_STRING,
        _('String EOL')    : GTKSCINTILLA.LISP_STRING_EOL,
        _('Identifier')    : GTKSCINTILLA.LISP_IDENTIFIER,
        _('Operator')      : GTKSCINTILLA.LISP_OPERATOR },
    GTKSCINTILLA.LEXER_RUBY : {
        _('White space')               : GTKSCINTILLA.RUBY_WHITE_SPACE,
        _('Comment line')              : GTKSCINTILLA.RUBY_COMMENT_LINE,
        _('Number')                    : GTKSCINTILLA.RUBY_NUMBER,
        _('String double quote')       : GTKSCINTILLA.RUBY_STRING_DOUBLE,
        _('String single quote')       : GTKSCINTILLA.RUBY_STRING_SINGLE,
        _('Keyword')                   : GTKSCINTILLA.RUBY_KEYWORD,
        _('Long string double quote')  : GTKSCINTILLA.RUBY_TRIPLE_DOUBLE,
        _('Long string single quote')  : GTKSCINTILLA.RUBY_TRIPLE_SINGLE,
        _('Class name definition')     : GTKSCINTILLA.RUBY_CLASS_STATEMENT,
        _('Function name definition')  : GTKSCINTILLA.RUBY_DEF_STATEMENT,
        _('Operator')                  : GTKSCINTILLA.RUBY_OPERATOR,
        _('Identifier')                : GTKSCINTILLA.RUBY_IDENTIFIER,
        _('Comment block')             : GTKSCINTILLA.RUBY_COMMENT_BLOCK,
        _('String EOL')                : GTKSCINTILLA.RUBY_STRING_EOL },
    GTKSCINTILLA.LEXER_EIFFEL : {
    },
    GTKSCINTILLA.LEXER_EIFFELKW : {
    },
    GTKSCINTILLA.LEXER_NNCRONTAB : {
    },
    GTKSCINTILLA.LEXER_SQL : {
        _('White space')           : GTKSCINTILLA.SQL_WHITE_SPACE,
        _('Comment block')         : GTKSCINTILLA.SQL_COMMENT_BLOCK,
        _('Comment line')          : GTKSCINTILLA.SQL_COMMENT_LINE,
        _('Comment doc')           : GTKSCINTILLA.SQL_COMMENT_DOC,
        _('Number')                : GTKSCINTILLA.SQL_NUMBER,
        _('Keyword')               : GTKSCINTILLA.SQL_KEYWORD,
        _('String double quote')   : GTKSCINTILLA.SQL_DOUBLE_STRING,
        _('String single quote')   : GTKSCINTILLA.SQL_SINGLE_STRING,
        _('Symbols')               : GTKSCINTILLA.SQL_SYMBOLS,
        _('Preprocessor')          : GTKSCINTILLA.SQL_PREPROCESSOR,
        _('Operator')              : GTKSCINTILLA.SQL_OPERATOR,
        _('Identifier')            : GTKSCINTILLA.SQL_IDENTIFIER,
        _('String EOL')            : GTKSCINTILLA.SQL_STRING_EOL },
    GTKSCINTILLA.LEXER_VB : {
        _('White space')       : GTKSCINTILLA.VB_WHITE_SPACE,
        _('Comment block')     : GTKSCINTILLA.VB_COMMENT_BLOCK,
        _('Comment line')      : GTKSCINTILLA.VB_COMMENT_LINE,
        _('Comment doc')       : GTKSCINTILLA.VB_COMMENT_DOC,
        _('Number')            : GTKSCINTILLA.VB_NUMBER,
        _('Keyword')           : GTKSCINTILLA.VB_WORD,
        _('String')            : GTKSCINTILLA.VB_STRING,
        _('Character')         : GTKSCINTILLA.VB_CHARACTER,
        _('UUID')              : GTKSCINTILLA.VB_UUID,
        _('Preprocessor')      : GTKSCINTILLA.VB_PREPROCESSOR,
        _('Operator')          : GTKSCINTILLA.VB_OPERATOR,
        _('Identifier')        : GTKSCINTILLA.VB_IDENTIFIER,
        _('String EOL')        : GTKSCINTILLA.VB_STRING_EOL },
    GTKSCINTILLA.LEXER_PROPERTIES : {
        _('White space')       : GTKSCINTILLA.PROPERTIES_DEFAULT,
        _('Comment')           : GTKSCINTILLA.PROPERTIES_COMMENT,
        _('Section')           : GTKSCINTILLA.PROPERTIES_SECTION,
        _('Assignment')        : GTKSCINTILLA.PROPERTIES_OPERATOR,
        _('Default value (@)') : GTKSCINTILLA.PROPERTIES_DEFAULT_VAL
        },
    GTKSCINTILLA.LEXER_DIFF : {
        _('Context lines') : GTKSCINTILLA.DIFF_DEFAULT,
        _('Comment')       : GTKSCINTILLA.DIFF_COMMENT,
        _('Command')       : GTKSCINTILLA.DIFF_COMMAND,
        _('File name')     : GTKSCINTILLA.DIFF_FILES,
        _('Position')      : GTKSCINTILLA.DIFF_POSITION,
        _('Line removed')  : GTKSCINTILLA.DIFF_DEL_LINE,
        _('Line added')    : GTKSCINTILLA.DIFF_ADD_LINE },
    GTKSCINTILLA.LEXER_PASCAL : {
        _('White space')           : GTKSCINTILLA.PASCAL_WHITE_SPACE,
        _('Comment block')         : GTKSCINTILLA.PASCAL_COMMENT_BLOCK,
        _('Comment line')          : GTKSCINTILLA.PASCAL_COMMENT_LINE,
        _('Comment doc')           : GTKSCINTILLA.PASCAL_COMMENT_DOC,
        _('Number')                : GTKSCINTILLA.PASCAL_NUMBER,
        _('Keyword')               : GTKSCINTILLA.PASCAL_KEYWORD,
        _('String double quote')   : GTKSCINTILLA.PASCAL_DOUBLE_STRING,
        _('String single quote')   : GTKSCINTILLA.PASCAL_SINGLE_STRING,
        _('Symbols')               : GTKSCINTILLA.PASCAL_SYMBOLS,
        _('Preprocessor')          : GTKSCINTILLA.PASCAL_PREPROCESSOR },
}

def _merge_font(font):
    """Merge the given X font descripton with the deafult font"""
    df = string.split(Moleskine.app.prefs['default style/font'], '-')
    sf = string.split(font, '-')
    for i in range(min(len(df), len(sf))):
        if sf[i] == '*':
            sf[i] = df[i]
    return string.join(sf, '-')

def _split_font(font):
    """Split the given X font descripton from the deafult font
    
    If the given font is equivalent to the default, the function returns None.
    """
    df = string.split(Moleskine.app.prefs['default style/font'], '-')
    sf = string.split(font, '-')
    count = 0
    for i in range(min(len(df), len(sf))):
        if sf[i] == df[i]:
            sf[i] = '*'
            count = count + 1
    if count == min(len(df), len(sf)):
        return None
    else:
        return string.join(sf, '-')[1:]

class Language:
    def __init__(self, name, lexer):
        self.name = name
        if lexer is None:
            self.load()
        else:
            self.lexer = lexer
            self.init_lang()
            self.init_styles()
    
    def init_lang(self):
        self.keywords = ['', '', '', '', '']
        self.tab_size = 4
        self.soft_tabs = 1
        self.indent_size = 4
        self.show_indent = 1
        self.auto_indent = 1
        self.edge_indicator = 0
        self.edge_column = 80
        self.edge_color = 0
        self.rules = []

    def init_styles(self):
        self.styles = {}
        if not Styles.has_key(self.lexer):
            self.styles[0] = {
                'font': None,
                'fore': None,
                'back': None }
            return
        for i in Styles[self.lexer].values():
            self.styles[i] = {
                'font': None,
                'fore': None,
                'back': None }
    
    def load(self):
        gnome.config.push_prefix(os.path.join(_CONFIG_FILE, self.name) + '/')
        
        self.lexer = gnome.config.get_int('lexer')
        self.api_file = gnome.config.get_string('api_file')
        self.keywords = [None, None, None, None, None]
        self.keywords[0] = gnome.config.get_string('keywords1')
        self.keywords[1] = gnome.config.get_string('keywords2')
        self.keywords[2] = gnome.config.get_string('keywords3')
        self.keywords[3] = gnome.config.get_string('keywords4')
        self.keywords[4] = gnome.config.get_string('keywords5')
        self.tab_size = gnome.config.get_int('tab_size')
        self.soft_tabs = gnome.config.get_bool('soft_tabs')
        self.indent_size = gnome.config.get_int('indent_size')
        self.show_indent = gnome.config.get_bool('show_indent')
        self.auto_indent = gnome.config.get_bool('auto_indent')
        self.edge_indicator = gnome.config.get_int('edge_indicator')
        self.edge_column = gnome.config.get_int('edge_column')
        self.edge_color = gnome.config.get_int('edge_color')
        
        self.load_styles()
        self.load_rules()
        
        gnome.config.pop_prefix()
    
    def load_rules(self):
        self.rules = []
        rules_count = gnome.config.get_int('rules_count')
        for i in range(rules_count):
            type = gnome.config.get_int('type_%i_type' % i)
            rule = gnome.config.get_string('type_%i_rule' % i)
            self.rules.append((type, rule))
    
    def load_styles(self):
        self.init_styles()
        
        if not Styles.has_key(self.lexer):
            return
        
        for style in Styles[self.lexer].values():
            s = self.styles[style]
            s['font'] = gnome.config.get_string('style_%i_font' % style)
            s['fore'] = gnome.config.get_string('style_%i_fore' % style)
            s['back'] = gnome.config.get_string('style_%i_back' % style)
    
    def save(self):
        gnome.config.push_prefix(os.path.join(_CONFIG_FILE, self.name) + '/')
        
        gnome.config.set_int('lexer', self.lexer)
        if self.api_file is not None:
            gnome.config.set_string('api_file', self.api_file)
        else:
            gnome.config.set_string('api_file', '')
        if self.keywords[0] is not None:
            gnome.config.set_string('keywords1', self.keywords[0])
        if self.keywords[1] is not None:
            gnome.config.set_string('keywords2', self.keywords[1])
        if self.keywords[2] is not None:
            gnome.config.set_string('keywords3', self.keywords[2])
        if self.keywords[3] is not None:
            gnome.config.set_string('keywords4', self.keywords[3])
        if self.keywords[4] is not None:
            gnome.config.set_string('keywords5', self.keywords[4])
        gnome.config.set_int('tab_size', self.tab_size)
        gnome.config.set_bool('soft_tabs', self.soft_tabs)
        gnome.config.set_int('indent_size', self.indent_size)
        gnome.config.set_bool('show_indent', self.show_indent)
        gnome.config.set_bool('auto_indent', self.auto_indent)
        gnome.config.set_int('edge_indicator', self.edge_indicator)
        gnome.config.set_int('edge_column', self.edge_column)
        gnome.config.set_int('edge_color', self.edge_color)
        
        self.save_styles()
        self.save_rules()
        
        gnome.config.pop_prefix()
    
    def save_rules(self):
        gnome.config.set_int('rules_count', len(self.rules))
        for i in range(len(self.rules)):
            gnome.config.set_int('type_%i_type' % i, self.rules[i][0])
            gnome.config.set_string('type_%i_rule' % i, self.rules[i][1])
    
    def save_styles(self):
        if not Styles.has_key(self.lexer):
            return
        
        for style in self.styles.keys():
            s = self.styles[style]
            if s['font'] is not None:
                if Moleskine.app.prefs['default style/font'] != s['font']:
                    gnome.config.set_string('style_%i_font' % style, s['font'])
                else:
                    s['font'] = None
            if s['fore'] is not None:
                if Moleskine.app.prefs['default style/fore'] != s['fore']:
                    gnome.config.set_string('style_%i_fore' % style, s['fore'])
                else:
                    s['fore'] = None
            if s['back'] is not None:
                if Moleskine.app.prefs['default style/back'] != s['back']:
                    gnome.config.set_string('style_%i_back' % style, s['back'])
                else:
                    s['back'] = None

class LanguagesList(UserDict):
    def new(self, name, lexer = None):
        self.data[name] = Language(name, lexer)
    
    def remove(self, name):
        del self.data[name]
    
    def load(self):
        gnome.config.push_prefix(_CONFIG_FILE)
        stock_langs = gnome.config.get_string('Languages/stock_languages')
        langs = gnome.config.get_string('Languages/languages')
        gnome.config.pop_prefix()
        
        langs_list = []
        if langs is not None:
            langs_list = string.split(langs, '\n')
        stock_langs_list = []
        if stock_langs is not None:
            stock_langs_list = string.split(stock_langs, '\n')
        
        d = {}
        for i in langs_list:
            d[i] = None
        for i in stock_langs_list:
            d[i] = None
        langs_list = d.keys()
        
        for lang_name in langs_list:
            self.new(lang_name)
        
        gnome.config.drop_all()
    
    def save(self):
        gnome.config.clean_file(_CONFIG_FILE)
        gnome.config.push_prefix(_CONFIG_FILE)
        langs = string.join(self.keys(), '\n')
        gnome.config.set_string('Languages/languages', langs)
        gnome.config.pop_prefix()
        
        for lang in self.values():
            lang.save()
        
        gnome.config.sync()
        gnome.config.drop_all()



# Config.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2000 - 2002   Michele Campeotto <micampe@micampe.it>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from UserDict import UserDict
import os.path
import string

import gnome.config

class Config(UserDict):
    def __init__(self, prefix):
        UserDict.__init__(self)
        if prefix[-1] != '/':
            prefix = prefix + '/'
        if prefix[0] == '/':
            prefix = prefix[1:]
        self.prefix = prefix
    
    def __setitem__(self, key, item):
        if not '/' in key:
            raise KeyError(_('Keys must be in the form "section/key"'))
        UserDict.__setitem__(self, key, item)
    
    def __getitem__(self, key):
        if not '/' in key:
            raise KeyError(_('Keys must be in the form "section/key"'))
        if not self.data.has_key(key):
            gnome.config.push_prefix(self.prefix)
            self.data[key] = gnome.config.get_string(key)
            if self.data[key] == 'None':
                self.data[key] = None
            gnome.config.pop_prefix()
            gnome.config.drop_file(self.prefix)
        return self.data[key]
    
    def has_key(self, key):
        if not '/' in key:
            raise KeyError(_('Keys must be in the form "section/key"'))
        return UserDict.has_key(self, key)
    
    def save(self):
        gnome.config.clean_file(self.prefix)
        gnome.config.push_prefix(self.prefix)
        
        for k, v in self.data.items():
            gnome.config.set_string(k, str(v))
        
        gnome.config.pop_prefix()
        gnome.config.sync()
        gnome.config.drop_file(self.prefix)
